from _typeshed import Incomplete
from abc import abstractmethod
from collections.abc import Mapping, Sequence
from typing import Any, Literal, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import NotRequired

from ._distutils.cmd import Command as _Command
from .command.alias import alias
from .command.bdist_egg import bdist_egg
from .command.bdist_rpm import bdist_rpm
from .command.bdist_wheel import bdist_wheel
from .command.build import build
from .command.build_clib import build_clib
from .command.build_ext import build_ext
from .command.build_py import build_py
from .command.develop import develop
from .command.dist_info import dist_info
from .command.easy_install import easy_install
from .command.editable_wheel import editable_wheel
from .command.egg_info import egg_info
from .command.install import install
from .command.install_egg_info import install_egg_info
from .command.install_lib import install_lib
from .command.install_scripts import install_scripts
from .command.rotate import rotate
from .command.saveopts import saveopts
from .command.sdist import sdist
from .command.setopt import setopt
from .depends import Require as Require
from .discovery import _Finder
from .dist import Distribution as Distribution
from .extension import Extension as Extension
from .warnings import SetuptoolsDeprecationWarning as SetuptoolsDeprecationWarning

_CommandT = TypeVar("_CommandT", bound=_Command)

__all__ = [
    "setup",
    "Distribution",
    "Command",
    "Extension",
    "Require",
    "SetuptoolsDeprecationWarning",
    "find_packages",
    "find_namespace_packages",
]

__version__: str

@type_check_only
class _BuildInfo(TypedDict):
    sources: list[str] | tuple[str, ...]
    obj_deps: NotRequired[dict[str, list[str] | tuple[str, ...]]]
    macros: NotRequired[list[tuple[str] | tuple[str, str | None]]]
    include_dirs: NotRequired[list[str]]
    cflags: NotRequired[list[str]]

find_packages = _Finder.find
find_namespace_packages = _Finder.find

def setup(
    *,
    name: str = ...,
    version: str = ...,
    description: str = ...,
    long_description: str = ...,
    long_description_content_type: str = ...,
    author: str = ...,
    author_email: str = ...,
    maintainer: str = ...,
    maintainer_email: str = ...,
    url: str = ...,
    download_url: str = ...,
    packages: list[str] = ...,
    py_modules: list[str] = ...,
    scripts: list[str] = ...,
    ext_modules: Sequence[Extension] = ...,
    classifiers: list[str] = ...,
    distclass: type[Distribution] = ...,
    script_name: str = ...,
    script_args: list[str] = ...,
    options: Mapping[str, Incomplete] = ...,
    license: str = ...,
    keywords: list[str] | str = ...,
    platforms: list[str] | str = ...,
    cmdclass: Mapping[str, type[_Command]] = ...,
    data_files: list[tuple[str, list[str]]] = ...,
    package_dir: Mapping[str, str] = ...,
    obsoletes: list[str] = ...,
    provides: list[str] = ...,
    requires: list[str] = ...,
    command_packages: list[str] = ...,
    command_options: Mapping[str, Mapping[str, tuple[Incomplete, Incomplete]]] = ...,
    package_data: Mapping[str, list[str]] = ...,
    include_package_data: bool = ...,
    # libraries for `Distribution` or `build_clib`, not `Extension`, `build_ext` or `CCompiler`
    libraries: list[tuple[str, _BuildInfo]] = ...,
    headers: list[str] = ...,
    ext_package: str = ...,
    include_dirs: list[str] = ...,
    password: str = ...,
    fullname: str = ...,
    # Custom Distributions could accept more params
    **attrs: Any,
) -> Distribution: ...

class Command(_Command):
    command_consumes_arguments: bool
    distribution: Distribution
    # Any: Dynamic command subclass attributes
    def __init__(self, dist: Distribution, **kw: Any) -> None: ...
    # Note: Commands that setuptools doesn't re-expose are considered deprecated (they must be imported from distutils directly)
    # So we're not listing them here. This list comes directly from the setuptools/command folder. Minus the test command.
    @overload  # type: ignore[override]
    def get_finalized_command(self, command: Literal["alias"], create: bool | Literal[0, 1] = 1) -> alias: ...
    @overload
    def get_finalized_command(self, command: Literal["bdist_egg"], create: bool | Literal[0, 1] = 1) -> bdist_egg: ...
    @overload
    def get_finalized_command(self, command: Literal["bdist_rpm"], create: bool | Literal[0, 1] = 1) -> bdist_rpm: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["bdist_wheel"], create: bool | Literal[0, 1] = 1) -> bdist_wheel: ...
    @overload
    def get_finalized_command(self, command: Literal["build"], create: bool | Literal[0, 1] = 1) -> build: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_clib"], create: bool | Literal[0, 1] = 1) -> build_clib: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_ext"], create: bool | Literal[0, 1] = 1) -> build_ext: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["build_py"], create: bool | Literal[0, 1] = 1) -> build_py: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["develop"], create: bool | Literal[0, 1] = 1) -> develop: ...
    @overload
    def get_finalized_command(self, command: Literal["dist_info"], create: bool | Literal[0, 1] = 1) -> dist_info: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["easy_install"], create: bool | Literal[0, 1] = 1) -> easy_install: ...
    @overload
    def get_finalized_command(self, command: Literal["editable_wheel"], create: bool | Literal[0, 1] = 1) -> editable_wheel: ...
    @overload
    def get_finalized_command(self, command: Literal["egg_info"], create: bool | Literal[0, 1] = 1) -> egg_info: ...
    @overload
    def get_finalized_command(self, command: Literal["install"], create: bool | Literal[0, 1] = 1) -> install: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(
        self, command: Literal["install_egg_info"], create: bool | Literal[0, 1] = 1
    ) -> install_egg_info: ...
    @overload
    def get_finalized_command(self, command: Literal["install_lib"], create: bool | Literal[0, 1] = 1) -> install_lib: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["install_scripts"], create: bool | Literal[0, 1] = 1) -> install_scripts: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["rotate"], create: bool | Literal[0, 1] = 1) -> rotate: ...
    @overload
    def get_finalized_command(self, command: Literal["saveopts"], create: bool | Literal[0, 1] = 1) -> saveopts: ...
    @overload
    def get_finalized_command(self, command: Literal["sdist"], create: bool | Literal[0, 1] = 1) -> sdist: ...  # type: ignore[overload-overlap]
    @overload
    def get_finalized_command(self, command: Literal["setopt"], create: bool | Literal[0, 1] = 1) -> setopt: ...
    @overload
    def get_finalized_command(self, command: str, create: bool | Literal[0, 1] = 1) -> Command: ...
    @overload  # type: ignore[override] # Extra **kw param
    def reinitialize_command(self, command: Literal["alias"], reinit_subcommands: bool = False, **kw) -> alias: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_egg"], reinit_subcommands: bool = False, **kw) -> bdist_egg: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_rpm"], reinit_subcommands: bool = False, **kw) -> bdist_rpm: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_wheel"], reinit_subcommands: bool = False, **kw) -> bdist_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["build"], reinit_subcommands: bool = False, **kw) -> build: ...
    @overload
    def reinitialize_command(self, command: Literal["build_clib"], reinit_subcommands: bool = False, **kw) -> build_clib: ...
    @overload
    def reinitialize_command(self, command: Literal["build_ext"], reinit_subcommands: bool = False, **kw) -> build_ext: ...
    @overload
    def reinitialize_command(self, command: Literal["build_py"], reinit_subcommands: bool = False, **kw) -> build_py: ...
    @overload
    def reinitialize_command(self, command: Literal["develop"], reinit_subcommands: bool = False, **kw) -> develop: ...
    @overload
    def reinitialize_command(self, command: Literal["dist_info"], reinit_subcommands: bool = False, **kw) -> dist_info: ...
    @overload
    def reinitialize_command(self, command: Literal["easy_install"], reinit_subcommands: bool = False, **kw) -> easy_install: ...
    @overload
    def reinitialize_command(
        self, command: Literal["editable_wheel"], reinit_subcommands: bool = False, **kw
    ) -> editable_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["egg_info"], reinit_subcommands: bool = False, **kw) -> egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install"], reinit_subcommands: bool = False, **kw) -> install: ...
    @overload
    def reinitialize_command(
        self, command: Literal["install_egg_info"], reinit_subcommands: bool = False, **kw
    ) -> install_egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install_lib"], reinit_subcommands: bool = False, **kw) -> install_lib: ...
    @overload
    def reinitialize_command(
        self, command: Literal["install_scripts"], reinit_subcommands: bool = False, **kw
    ) -> install_scripts: ...
    @overload
    def reinitialize_command(self, command: Literal["rotate"], reinit_subcommands: bool = False, **kw) -> rotate: ...
    @overload
    def reinitialize_command(self, command: Literal["saveopts"], reinit_subcommands: bool = False, **kw) -> saveopts: ...
    @overload
    def reinitialize_command(self, command: Literal["sdist"], reinit_subcommands: bool = False, **kw) -> sdist: ...
    @overload
    def reinitialize_command(self, command: Literal["setopt"], reinit_subcommands: bool = False, **kw) -> setopt: ...
    @overload
    def reinitialize_command(self, command: str, reinit_subcommands: bool = False, **kw) -> Command: ...
    @overload
    def reinitialize_command(self, command: _CommandT, reinit_subcommands: bool = False, **kw) -> _CommandT: ...
    @abstractmethod
    def initialize_options(self) -> None: ...
    @abstractmethod
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...

class sic(str): ...



import re
from typing import ClassVar, Final

from docutils import nodes
from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class Headers(Transform):
    default_priority: ClassVar[int]
    pep_url: ClassVar[str]
    pep_cvs_url: ClassVar[str]
    rcs_keyword_substitutions: ClassVar[tuple[tuple[re.Pattern[str], str], ...]]
    def apply(self) -> None: ...

class Contents(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class TargetNotes(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...
    def cleanup_callback(self, pending: nodes.pending) -> None: ...

class PEPZero(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class PEPZeroSpecial(nodes.SparseNodeVisitor):
    pep_url: ClassVar[str]
    def unknown_visit(self, node: nodes.Node) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    pep_table: bool
    entry: int
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...

non_masked_addresses: tuple[str, ...]

def mask_email(ref: nodes.reference, pepno: int | None = None) -> nodes.Node: ...



from __future__ import annotations

from urllib.parse import quote, quote_plus, urlencode

urlencode({"a": "b"}, quote_via=quote)
urlencode({b"a": b"b"}, quote_via=quote)
urlencode({"a": b"b"}, quote_via=quote)
urlencode({b"a": "b"}, quote_via=quote)
mixed_dict: dict[str | bytes, str | bytes] = {}
urlencode(mixed_dict, quote_via=quote)

urlencode({"a": "b"}, quote_via=quote_plus)



from typing import ClassVar, Final

from docutils import writers

__docformat__: Final = "reStructuredText"

class Writer(writers.UnfilteredWriter[str]):
    supported: ClassVar[tuple[str, ...]]
    config_section: ClassVar[str]
    config_section_dependencies: ClassVar[tuple[str]]
    def translate(self) -> None: ...



from collections.abc import Sequence
from typing import Literal, SupportsIndex, overload
from typing_extensions import TypeAlias

import numpy as np
from numpy.typing import NDArray

from ._enum import ParamEnum
from ._geometry import GeometryType
from ._typing import ArrayLike, ArrayLikeSeq, GeoArray, OptGeoArrayLike, OptGeoArrayLikeSeq
from .geometry import GeometryCollection, LinearRing, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon
from .lib import Geometry

__all__ = [
    "box",
    "destroy_prepared",
    "empty",
    "geometrycollections",
    "linearrings",
    "linestrings",
    "multilinestrings",
    "multipoints",
    "multipolygons",
    "points",
    "polygons",
    "prepare",
]

class HandleNaN(ParamEnum):
    allow = 0
    skip = 1
    error = 2

_HandleNaN: TypeAlias = Literal["allow", "skip", "error"] | HandleNaN

@overload
def points(
    coords: float,
    y: float,
    z: float | None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,  # acts as x
) -> Point: ...
@overload
def points(
    coords: Sequence[float],
    y: None = None,
    z: None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,  # acts as x, y[, z]
) -> Point: ...
@overload
def points(
    coords: Sequence[float],  # acts as (x1, x2, ...)
    y: Sequence[float],  # must be (y1, y2, ...)
    z: Sequence[float] | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def points(
    coords: Sequence[Sequence[float]],  # acts as (x1, x2, ...), (y1, y2, ...)[, (z1, z2, ...)]
    y: None = None,
    z: None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def points(
    coords: ArrayLike[float],
    y: ArrayLike[float],
    z: ArrayLike[float] | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> Point | GeoArray: ...
@overload
def points(
    coords: ArrayLikeSeq[float],
    y: ArrayLike[float] | None = None,
    z: ArrayLike[float] | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> Point | GeoArray: ...
@overload
def linestrings(
    coords: Sequence[float],  # acts as (x1, x2, ...)
    y: Sequence[float],
    z: Sequence[float] | None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,
) -> LineString: ...
@overload
def linestrings(
    coords: Sequence[Sequence[float]],  # acts as (x1, y1[, z1]), (x2, y2[, z2]), ...
    y: None = None,
    z: None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,
) -> LineString: ...
@overload
def linestrings(
    coords: Sequence[Sequence[Sequence[float]]],  # acts as seq of (x1, y1[, z1]), (x2, y2[, z2]), ...
    y: None = None,
    z: None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def linestrings(
    coords: ArrayLikeSeq[float],
    y: ArrayLikeSeq[float] | None = None,
    z: ArrayLikeSeq[float] | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> LineString | GeoArray: ...
@overload
def linearrings(
    coords: Sequence[float],  # acts as (x1, x2, ...)
    y: Sequence[float],
    z: Sequence[float] | None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,
) -> LinearRing: ...
@overload
def linearrings(
    coords: Sequence[Sequence[float]],  # acts as (x1, y1[, z1]), (x2, y2[, z2]), ...
    y: None = None,
    z: None = None,
    indices: None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: None = None,
    **kwargs,
) -> LinearRing: ...
@overload
def linearrings(
    coords: Sequence[Sequence[Sequence[float]]],  # acts as seq of (x1, y1[, z1]), (x2, y2[, z2]), ...
    y: None = None,
    z: None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def linearrings(
    coords: ArrayLikeSeq[float],
    y: ArrayLikeSeq[float] | None = None,
    z: ArrayLikeSeq[float] | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    handle_nan: _HandleNaN = ...,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> LinearRing | GeoArray: ...
@overload
def polygons(
    geometries: LinearRing | Sequence[Sequence[float]] | None,
    holes: ArrayLikeSeq[float] | OptGeoArrayLikeSeq | None = None,
    indices: None = None,
    *,
    out: None = None,
    **kwargs,
) -> Polygon: ...
@overload
def polygons(
    geometries: Sequence[LinearRing | Sequence[Sequence[float]] | None],
    holes: ArrayLikeSeq[float] | OptGeoArrayLikeSeq | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def polygons(
    geometries: ArrayLikeSeq[float] | OptGeoArrayLikeSeq,
    holes: ArrayLikeSeq[float] | OptGeoArrayLikeSeq | None = None,
    indices: ArrayLikeSeq[int] | None = None,
    *,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> Polygon | GeoArray: ...
@overload
def box(xmin: float, ymin: float, xmax: float, ymax: float, ccw: bool = True, **kwargs) -> Polygon: ...
@overload
def box(
    xmin: ArrayLikeSeq[float],
    ymin: ArrayLikeSeq[float],
    xmax: ArrayLikeSeq[float],
    ymax: ArrayLikeSeq[float],
    ccw: bool = True,
    **kwargs,
) -> GeoArray: ...
@overload
def multipoints(
    geometries: Sequence[Point | Sequence[float] | None], indices: None = None, *, out: None = None, **kwargs
) -> MultiPoint: ...
@overload
def multipoints(
    geometries: Sequence[Sequence[Point | Sequence[float] | None]],
    indices: ArrayLikeSeq[int] | None = None,
    *,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def multipoints(
    geometries: OptGeoArrayLikeSeq, indices: ArrayLikeSeq[int] | None = None, *, out: NDArray[np.object_] | None = None, **kwargs
) -> MultiPoint | GeoArray: ...
@overload
def multilinestrings(
    geometries: Sequence[LineString | Sequence[Sequence[float]] | None], indices: None = None, *, out: None = None, **kwargs
) -> MultiLineString: ...
@overload
def multilinestrings(
    geometries: Sequence[Sequence[LineString | Sequence[Sequence[float]] | None]],
    indices: ArrayLikeSeq[int] | None = None,
    *,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def multilinestrings(
    geometries: OptGeoArrayLikeSeq, indices: ArrayLikeSeq[int] | None = None, *, out: NDArray[np.object_] | None = None, **kwargs
) -> MultiLineString | GeoArray: ...
@overload
def multipolygons(
    geometries: Sequence[Polygon | Sequence[Sequence[float]] | None], indices: None = None, *, out: None = None, **kwargs
) -> MultiPolygon: ...
@overload
def multipolygons(
    geometries: Sequence[Sequence[Polygon | Sequence[Sequence[float]] | None]],
    indices: ArrayLikeSeq[int] | None = None,
    *,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def multipolygons(
    geometries: OptGeoArrayLikeSeq, indices: ArrayLikeSeq[int] | None = None, *, out: NDArray[np.object_] | None = None, **kwargs
) -> MultiPolygon | GeoArray: ...
@overload
def geometrycollections(
    geometries: Sequence[Geometry | None], indices: None = None, out: None = None, **kwargs
) -> GeometryCollection: ...
@overload
def geometrycollections(
    geometries: Sequence[Sequence[Geometry | None]],
    indices: ArrayLikeSeq[int] | None = None,
    out: NDArray[np.object_] | None = None,
    **kwargs,
) -> GeoArray: ...
@overload
def geometrycollections(
    geometries: OptGeoArrayLikeSeq, indices: ArrayLikeSeq[int] | None = None, out: NDArray[np.object_] | None = None, **kwargs
) -> GeometryCollection | GeoArray: ...
def prepare(geometry: OptGeoArrayLike, **kwargs) -> None: ...
def destroy_prepared(geometry: OptGeoArrayLike, **kwargs) -> None: ...
def empty(
    shape: SupportsIndex | Sequence[SupportsIndex], geom_type: GeometryType | int | None = None, order: Literal["C", "F"] = "C"
) -> NDArray[np.object_]: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Final
from typing_extensions import TypeAlias

import grpc
import grpc.aio
from google.protobuf import descriptor_pool
from grpc_reflection.v1alpha import reflection_pb2 as _reflection_pb2
from grpc_reflection.v1alpha._base import BaseReflectionServicer

from . import _async as aio

SERVICE_NAME: Final[str]

_AnyServer: TypeAlias = grpc.Server | grpc.aio.Server
_AnyServicerContext: TypeAlias = grpc.ServicerContext | grpc.aio.ServicerContext[Incomplete, Incomplete]

class ReflectionServicer(BaseReflectionServicer):
    def ServerReflectionInfo(
        self, request_iterator: Iterable[_reflection_pb2.ServerReflectionRequest], context: _AnyServicerContext
    ): ...

def enable_server_reflection(
    service_names: Iterable[str], server: _AnyServer, pool: descriptor_pool.DescriptorPool | None = None
) -> None: ...

__all__ = ["SERVICE_NAME", "ReflectionServicer", "enable_server_reflection", "aio"]



from .reference import Reference

def SeriesFactory(
    values: Reference | str,
    xvalues: Reference | str | None = None,
    zvalues: Reference | str | None = None,
    title: object = None,
    title_from_data: bool = False,
): ...



from collections.abc import Iterable
from re import Pattern

ALLOWED_KEY_TYPES: Iterable[str]
ALLOWED_EXPORT_KEY_TYPES: Iterable[str]
ALLOWED_DATA_KEY_TYPES: Iterable[str]
ALLOWED_DATA_KEY_BITS: Iterable[int]
ALLOWED_HASH_DATA_ALGORITHMS: Iterable[str]
ALLOWED_HASH_DATA_FORMATS: Iterable[str]
ALLOWED_SIGNATURE_ALGORITHMS: Iterable[str]
ALLOWED_MARSHALING_ALGORITHMS: Iterable[str]
ALLOWED_SALT_LENGTHS: Pattern[str]



from .authorization_code import AuthorizationCodeGrant as AuthorizationCodeGrant
from .base import (
    AuthorizationEndpointMixin as AuthorizationEndpointMixin,
    BaseGrant as BaseGrant,
    TokenEndpointMixin as TokenEndpointMixin,
)
from .client_credentials import ClientCredentialsGrant as ClientCredentialsGrant
from .implicit import ImplicitGrant as ImplicitGrant
from .refresh_token import RefreshTokenGrant as RefreshTokenGrant
from .resource_owner_password_credentials import ResourceOwnerPasswordCredentialsGrant as ResourceOwnerPasswordCredentialsGrant

__all__ = [
    "BaseGrant",
    "AuthorizationEndpointMixin",
    "TokenEndpointMixin",
    "AuthorizationCodeGrant",
    "ImplicitGrant",
    "ResourceOwnerPasswordCredentialsGrant",
    "ClientCredentialsGrant",
    "RefreshTokenGrant",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol buffer representing slices of a tensor"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TensorSliceProto(google.protobuf.message.Message):
    """Can only be interpreted if you know the corresponding TensorShape."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Extent(google.protobuf.message.Message):
        """Extent of the slice in one dimension.
        Either both or no attributes must be set.  When no attribute is set
        means: All data in that dimension.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        START_FIELD_NUMBER: builtins.int
        LENGTH_FIELD_NUMBER: builtins.int
        start: builtins.int
        """Start index of the slice, starting at 0."""
        length: builtins.int
        def __init__(self, *, start: builtins.int | None = ..., length: builtins.int | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["has_length", b"has_length", "length", b"length"]) -> builtins.bool: ...
        def ClearField(
            self, field_name: typing.Literal["has_length", b"has_length", "length", b"length", "start", b"start"]
        ) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["has_length", b"has_length"]) -> typing.Literal["length"] | None: ...

    EXTENT_FIELD_NUMBER: builtins.int
    @property
    def extent(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TensorSliceProto.Extent]:
        """Extent of the slice in all tensor dimensions.

        Must have one entry for each of the dimension of the tensor that this
        slice belongs to.  The order of sizes is the same as the order of
        dimensions in the TensorShape.
        """

    def __init__(self, *, extent: collections.abc.Iterable[global___TensorSliceProto.Extent] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["extent", b"extent"]) -> None: ...

global___TensorSliceProto = TensorSliceProto



from .exceptions import EOF as EOF, TIMEOUT as TIMEOUT, ExceptionPexpect as ExceptionPexpect
from .pty_spawn import spawn as spawn, spawnu as spawnu
from .run import run as run, runu as runu
from .utils import split_command_line as split_command_line, which as which

__version__: str
__revision__: str
__all__ = [
    "ExceptionPexpect",
    "EOF",
    "TIMEOUT",
    "spawn",
    "spawnu",
    "run",
    "runu",
    "which",
    "split_command_line",
    "__version__",
    "__revision__",
]



from _typeshed import Incomplete
from logging import Logger

from authlib.oauth2.rfc6749 import BaseGrant, TokenEndpointMixin

log: Logger
DEVICE_CODE_GRANT_TYPE: str

class DeviceCodeGrant(BaseGrant, TokenEndpointMixin):
    GRANT_TYPE = DEVICE_CODE_GRANT_TYPE
    TOKEN_ENDPOINT_AUTH_METHODS: Incomplete
    def validate_token_request(self) -> None: ...
    def create_token_response(self): ...
    def validate_device_credential(self, credential): ...
    def query_device_credential(self, device_code) -> None: ...
    def query_user_grant(self, user_code) -> None: ...
    def should_slow_down(self, credential) -> None: ...



#!/usr/bin/env python3
"""
Generates the protobuf stubs for the given protobuf version using mypy-protobuf.
Generally, new minor versions are a good time to update the stubs.
"""

from __future__ import annotations

import json
import re
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path
from typing import Any

from _utils import MYPY_PROTOBUF_VERSION, download_file, extract_archive, run_protoc
from ts_utils.metadata import read_metadata, update_metadata
from ts_utils.paths import distribution_path

# PyPi version has an extra "major" number that the Git version doesn't have
PACKAGE_VERSION = read_metadata("protobuf").version_spec.version[2:]

STUBS_FOLDER = distribution_path("protobuf").absolute()
ARCHIVE_FILENAME = f"protobuf-{PACKAGE_VERSION}.zip"
ARCHIVE_URL = f"https://github.com/protocolbuffers/protobuf/releases/download/v{PACKAGE_VERSION}/{ARCHIVE_FILENAME}"
EXTRACTED_PACKAGE_DIR = f"protobuf-{PACKAGE_VERSION}"

VERSION_PATTERN = re.compile(r'def game_version\(\):\n    return "(.+?)"')
PROTO_FILE_PATTERN = re.compile(r'"//:(.*)_proto"')


def extract_python_version(file_path: Path) -> str:
    """Extract the Python version from https://github.com/protocolbuffers/protobuf/blob/main/version.json ."""
    with file_path.open() as file:
        data: dict[str, Any] = json.load(file)
    # The root key will be the protobuf source code version
    version = next(iter(data.values()))["languages"]["python"]
    assert isinstance(version, str)
    return version


def extract_proto_file_paths(temp_dir: Path) -> list[str]:
    """
    Roughly reproduce the subset of .proto files on the public interface
    as described in py_proto_library calls in
    https://github.com/protocolbuffers/protobuf/blob/main/python/dist/BUILD.bazel .
    """
    with (temp_dir / EXTRACTED_PACKAGE_DIR / "python" / "dist" / "BUILD.bazel").open() as file:
        matched_lines = filter(None, (re.search(PROTO_FILE_PATTERN, line) for line in file))
        proto_files = [
            EXTRACTED_PACKAGE_DIR + "/src/google/protobuf/" + match.group(1).replace("compiler_", "compiler/") + ".proto"
            for match in matched_lines
        ]
    return proto_files


def main() -> None:
    temp_dir = Path(tempfile.mkdtemp())
    # Fetch s2clientprotocol (which contains all the .proto files)
    archive_path = temp_dir / ARCHIVE_FILENAME
    download_file(ARCHIVE_URL, archive_path)
    extract_archive(archive_path, temp_dir)

    # Remove existing pyi
    for old_stub in STUBS_FOLDER.rglob("*_pb2.pyi"):
        old_stub.unlink()

    protoc_version = run_protoc(
        proto_paths=(f"{EXTRACTED_PACKAGE_DIR}/src",),
        mypy_out=STUBS_FOLDER,
        proto_globs=extract_proto_file_paths(temp_dir),
        cwd=temp_dir,
    )

    python_protobuf_version = extract_python_version(temp_dir / EXTRACTED_PACKAGE_DIR / "version.json")

    # Cleanup after ourselves, this is a temp dir, but it can still grow fast if run multiple times
    shutil.rmtree(temp_dir)

    update_metadata(
        "protobuf",
        extra_description=f"""Partially generated using \
[mypy-protobuf=={MYPY_PROTOBUF_VERSION}](https://github.com/nipunn1313/mypy-protobuf/tree/v{MYPY_PROTOBUF_VERSION}) \
and {protoc_version} on \
[protobuf v{PACKAGE_VERSION}](https://github.com/protocolbuffers/protobuf/releases/tag/v{PACKAGE_VERSION}) \
(python `protobuf=={python_protobuf_version}`).""",
    )
    print("Updated protobuf/METADATA.toml")

    # Run pre-commit to cleanup the stubs
    subprocess.run((sys.executable, "-m", "pre_commit", "run", "--files", *STUBS_FOLDER.rglob("*_pb2.pyi")), check=False)


if __name__ == "__main__":
    main()



from collections.abc import Collection

def list_to_scope(scope: Collection[str] | str | None) -> str: ...
def scope_to_list(scope: Collection[str] | str | None) -> list[str]: ...
def extract_basic_authorization(headers: dict[str, str]) -> tuple[str, str]: ...



from _typeshed import Incomplete
from collections.abc import Generator, Iterable
from typing import TypeVar, overload

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

_U = TypeVar("_U")
__all__ = ["edge_boundary", "node_boundary"]

@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, dict[str, Incomplete]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, dict[str, Incomplete]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default: _U | None = None,
) -> Generator[tuple[_Node, _Node, dict[str, _U]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default: _U | None = None,
) -> Generator[tuple[_Node, _Node, dict[str, _U]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, int, dict[str, Incomplete]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default=None,
) -> Generator[tuple[_Node, _Node, int, dict[str, Incomplete]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default: _U | None = None,
) -> Generator[tuple[_Node, _Node, int, dict[str, _U]], None, None]: ...
@overload
def edge_boundary(
    G: Graph[_Node],
    nbunch1: Iterable[Incomplete],
    nbunch2: Iterable[Incomplete] | None = None,
    data=False,
    keys: bool = False,
    default: _U | None = None,
) -> Generator[tuple[_Node, _Node, int, dict[str, _U]], None, None]: ...
@_dispatchable
def node_boundary(G: Graph[_Node], nbunch1: Iterable[Incomplete], nbunch2: Iterable[Incomplete] | None = None) -> set[_Node]: ...



from _typeshed import Incomplete
from typing import ClassVar

from .setopt import option_base

def shquote(arg): ...

class alias(option_base):
    description: str
    command_consumes_arguments: bool
    user_options: ClassVar[list[tuple[str, str, str]]]
    boolean_options: ClassVar[list[str]]
    args: Incomplete
    remove: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...

def format_alias(name, aliases): ...



from collections.abc import Callable, Mapping
from typing import Any, ClassVar

from django.db.models import Model
from django.db.models.fields import NOT_PROVIDED

from .widgets import Widget

class Field:
    empty_values: ClassVar[list[str | None]]
    attribute: str | None
    default: type[NOT_PROVIDED] | Callable[[], Any] | Any
    column_name: str | None
    widget: Widget
    readonly: bool
    saves_null_values: bool
    dehydrate_method: str
    m2m_add: bool
    def __init__(
        self,
        attribute: str | None = None,
        column_name: str | None = None,
        widget: Widget | None = None,
        default: type[NOT_PROVIDED] | Callable[[], Any] | Any = ...,
        readonly: bool = False,
        saves_null_values: bool = True,
        dehydrate_method: str | None = None,
        m2m_add: bool = False,
    ) -> None: ...
    def clean(self, row: Mapping[str, Any], **kwargs: Any) -> Any: ...
    def get_value(self, instance: Model) -> Any: ...
    def save(self, instance: Model, row: Mapping[str, Any], is_m2m: bool = False, **kwargs: Any) -> None: ...
    def export(self, instance: Model, **kwargs: Any) -> str: ...
    def get_dehydrate_method(self, field_name: str | None = None) -> str: ...



import collections.abc
from collections.abc import Callable, Mapping
from re import Pattern
from typing import Any
from typing_extensions import Self, TypeAlias

from parsimonious.exceptions import ParseError
from parsimonious.grammar import Grammar
from parsimonious.nodes import Node
from parsimonious.utils import StrAndRepr

_CALLABLE_RETURN_TYPE: TypeAlias = int | tuple[int, list[Node]] | Node | None
_CALLABLE_TYPE: TypeAlias = (
    Callable[[str, int], _CALLABLE_RETURN_TYPE]
    | Callable[[str, int, Mapping[tuple[int, int], Node], ParseError, Grammar], _CALLABLE_RETURN_TYPE]
)

def is_callable(value: object) -> bool: ...
def expression(callable: _CALLABLE_TYPE, rule_name: str, grammar: Grammar) -> Expression: ...

IN_PROGRESS: object

class Expression(StrAndRepr):
    __slots__ = ["name", "identity_tuple"]
    name: str
    identity_tuple: tuple[str]
    def __init__(self, name: str = "") -> None: ...
    def resolve_refs(self, rule_map: Mapping[str, Expression]) -> Self: ...
    def parse(self, text: str, pos: int = 0) -> Node: ...
    def match(self, text: str, pos: int = 0) -> Node: ...
    def match_core(self, text: str, pos: int, cache: Mapping[tuple[int, int], Node], error: ParseError) -> Node: ...
    def as_rule(self) -> str: ...

class Literal(Expression):
    __slots__ = ["literal"]
    literal: str
    identity_tuple: tuple[str, str]  # type: ignore[assignment]
    def __init__(self, literal: str, name: str = "") -> None: ...

class TokenMatcher(Literal): ...

class Regex(Expression):
    __slots__ = ["re"]
    re: Pattern[str]
    identity_tuple: tuple[str, Pattern[str]]  # type: ignore[assignment]
    def __init__(
        self,
        pattern: str,
        name: str = "",
        ignore_case: bool = False,
        locale: bool = False,
        multiline: bool = False,
        dot_all: bool = False,
        unicode: bool = False,
        verbose: bool = False,
        ascii: bool = False,
    ) -> None: ...

class Compound(Expression):
    __slots__ = ["members"]
    members: collections.abc.Sequence[Expression]
    def __init__(self, *members: Expression, **kwargs: Any) -> None: ...

class Sequence(Compound): ...
class OneOf(Compound): ...

class Lookahead(Compound):
    __slots__ = ["negativity"]
    negativity: bool
    def __init__(self, member: Expression, *, negative: bool = False, **kwargs: Any) -> None: ...

def Not(term: Expression) -> Lookahead: ...

class Quantifier(Compound):
    __slots__ = ["min", "max"]
    min: int
    max: float
    def __init__(self, member: Expression, *, min: int = 0, max: float = ..., name: str = "", **kwargs: Any) -> None: ...

def ZeroOrMore(member: Expression, name: str = "") -> Quantifier: ...
def OneOrMore(member: Expression, name: str = "", min: int = 1) -> Quantifier: ...
def Optional(member: Expression, name: str = "") -> Quantifier: ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Jobs:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get(self, id: str) -> dict[str, Incomplete]: ...
    async def get_async(self, id: str) -> dict[str, Incomplete]: ...
    def get_failed_job(self, id: str) -> dict[str, Incomplete]: ...
    async def get_failed_job_async(self, id: str) -> dict[str, Incomplete]: ...
    def export_users(self, body: dict[str, Incomplete]): ...
    async def export_users_async(self, body: dict[str, Incomplete]): ...
    def import_users(
        self,
        connection_id: str,
        file_obj,
        upsert: bool = False,
        send_completion_email: bool = True,
        external_id: str | None = None,
    ) -> dict[str, Incomplete]: ...
    async def import_users_async(
        self,
        connection_id: str,
        file_obj,
        upsert: bool = False,
        send_completion_email: bool = True,
        external_id: str | None = None,
    ) -> dict[str, Incomplete]: ...
    def send_verification_email(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def send_verification_email_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete, Unused
from datetime import datetime
from typing import ClassVar, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Bool, DateTime, Float, Integer, NoneSet, Set, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedInteger
from openpyxl.descriptors.sequence import NestedSequence
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.pivot.fields import Error, Missing, Number, Text, TupleList
from openpyxl.pivot.table import PivotArea
from openpyxl.xml.functions import Element

from ..xml._functions_overloads import _HasTagAndGet

_RangePrGroupBy: TypeAlias = Literal["range", "seconds", "minutes", "hours", "days", "months", "quarters", "years"]
_CacheSourceType: TypeAlias = Literal["worksheet", "external", "consolidation", "scenario"]

class MeasureDimensionMap(Serialisable):
    tagname: ClassVar[str]
    measureGroup: Integer[Literal[True]]
    dimension: Integer[Literal[True]]
    def __init__(self, measureGroup: ConvertibleToInt | None = None, dimension: ConvertibleToInt | None = None) -> None: ...

class MeasureGroup(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[False]]
    caption: String[Literal[False]]
    def __init__(self, name: str, caption: str) -> None: ...

class PivotDimension(Serialisable):
    tagname: ClassVar[str]
    measure: Bool[Literal[False]]
    name: String[Literal[False]]
    uniqueName: String[Literal[False]]
    caption: String[Literal[False]]
    @overload
    def __init__(self, measure: _ConvertibleToBool = None, *, name: str, uniqueName: str, caption: str) -> None: ...
    @overload
    def __init__(self, measure: _ConvertibleToBool, name: str, uniqueName: str, caption: str) -> None: ...

class CalculatedMember(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[False]]
    mdx: String[Literal[False]]
    memberName: String[Literal[True]]
    hierarchy: String[Literal[True]]
    parent: String[Literal[True]]
    solveOrder: Integer[Literal[True]]
    set: Bool[Literal[False]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: str,
        mdx: str,
        memberName: str,
        hierarchy: str,
        parent: str,
        solveOrder: ConvertibleToInt,
        set: _ConvertibleToBool = None,
        extLst: Unused = None,
    ) -> None: ...

class CalculatedItem(Serialisable):
    tagname: ClassVar[str]
    field: Integer[Literal[True]]
    formula: String[Literal[False]]
    pivotArea: Typed[PivotArea, Literal[False]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(self, field: ConvertibleToInt | None = None, *, formula: str, pivotArea: PivotArea, extLst=None) -> None: ...
    @overload
    def __init__(self, field: ConvertibleToInt | None, formula: str, pivotArea: PivotArea, extLst=None) -> None: ...

class ServerFormat(Serialisable):
    tagname: ClassVar[str]
    culture: String[Literal[True]]
    format: String[Literal[True]]
    def __init__(self, culture: str | None = None, format: str | None = None) -> None: ...

class Query(Serialisable):
    tagname: ClassVar[str]
    mdx: String[Literal[False]]
    tpls: Typed[TupleList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, mdx: str, tpls: TupleList | None = None) -> None: ...

class OLAPSet(Serialisable):
    tagname: ClassVar[str]
    count: Integer[Literal[False]]
    maxRank: Integer[Literal[False]]
    setDefinition: String[Literal[False]]
    sortType: Incomplete
    queryFailed: Bool[Literal[False]]
    tpls: Typed[TupleList, Literal[True]]
    sortByTuple: Typed[TupleList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        count: ConvertibleToInt,
        maxRank: ConvertibleToInt,
        setDefinition: str,
        sortType=None,
        queryFailed: _ConvertibleToBool = None,
        tpls: TupleList | None = None,
        sortByTuple: TupleList | None = None,
    ) -> None: ...

class PCDSDTCEntries(Serialisable):
    tagname: ClassVar[str]
    count: Integer[Literal[True]]
    m: Typed[Missing, Literal[True]]
    n: Typed[Number, Literal[True]]
    e: Typed[Error, Literal[True]]
    s: Typed[Text, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, count: ConvertibleToInt, m: Missing, n: Number, e: Error, s: Text) -> None: ...

class TupleCache(Serialisable):
    tagname: ClassVar[str]
    entries: Typed[PCDSDTCEntries, Literal[True]]
    sets: NestedSequence[list[OLAPSet]]
    queryCache: NestedSequence[list[Query]]
    serverFormats: NestedSequence[list[ServerFormat]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        entries: PCDSDTCEntries | None = None,
        sets: list[OLAPSet] | tuple[OLAPSet, ...] = (),
        queryCache: list[Query] | tuple[Query, ...] = (),
        serverFormats: list[ServerFormat] | tuple[ServerFormat, ...] = (),
        extLst: ExtensionList | None = None,
    ) -> None: ...

class OLAPKPI(Serialisable):
    tagname: ClassVar[str]
    uniqueName: String[Literal[False]]
    caption: String[Literal[True]]
    displayFolder: String[Literal[True]]
    measureGroup: String[Literal[True]]
    parent: String[Literal[True]]
    value: String[Literal[False]]
    goal: String[Literal[True]]
    status: String[Literal[True]]
    trend: String[Literal[True]]
    weight: String[Literal[True]]
    time: String[Literal[True]]
    def __init__(
        self,
        uniqueName: str | None = None,
        caption: str | None = None,
        displayFolder: str | None = None,
        measureGroup: str | None = None,
        parent: str | None = None,
        value: str | None = None,
        goal: str | None = None,
        status: str | None = None,
        trend: str | None = None,
        weight: str | None = None,
        time: str | None = None,
    ) -> None: ...

class GroupMember(Serialisable):
    tagname: ClassVar[str]
    uniqueName: String[Literal[False]]
    group: Bool[Literal[False]]
    def __init__(self, uniqueName: str, group: _ConvertibleToBool = None) -> None: ...

class LevelGroup(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[False]]
    uniqueName: String[Literal[False]]
    caption: String[Literal[False]]
    uniqueParent: String[Literal[False]]
    id: Integer[Literal[False]]
    groupMembers: NestedSequence[list[GroupMember]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: str,
        uniqueName: str,
        caption: str,
        uniqueParent: str,
        id: ConvertibleToInt,
        groupMembers: list[GroupMember] | tuple[GroupMember, ...] = (),
    ) -> None: ...

class GroupLevel(Serialisable):
    tagname: ClassVar[str]
    uniqueName: String[Literal[False]]
    caption: String[Literal[False]]
    user: Bool[Literal[False]]
    customRollUp: Bool[Literal[False]]
    groups: NestedSequence[list[LevelGroup]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        uniqueName: str,
        caption: str,
        user: _ConvertibleToBool = None,
        customRollUp: _ConvertibleToBool = None,
        groups: list[LevelGroup] | tuple[LevelGroup, ...] = (),
        extLst: ExtensionList | None = None,
    ) -> None: ...

class FieldUsage(Serialisable):
    tagname: ClassVar[str]
    x: Integer[Literal[False]]
    def __init__(self, x: ConvertibleToInt) -> None: ...

class CacheHierarchy(Serialisable):
    tagname: ClassVar[str]
    uniqueName: String[Literal[False]]
    caption: String[Literal[True]]
    measure: Bool[Literal[False]]
    set: Bool[Literal[False]]
    parentSet: Integer[Literal[True]]
    iconSet: Integer[Literal[False]]
    attribute: Bool[Literal[False]]
    time: Bool[Literal[False]]
    keyAttribute: Bool[Literal[False]]
    defaultMemberUniqueName: String[Literal[True]]
    allUniqueName: String[Literal[True]]
    allCaption: String[Literal[True]]
    dimensionUniqueName: String[Literal[True]]
    displayFolder: String[Literal[True]]
    measureGroup: String[Literal[True]]
    measures: Bool[Literal[False]]
    count: Integer[Literal[False]]
    oneField: Bool[Literal[False]]
    memberValueDatatype: Integer[Literal[True]]
    unbalanced: Bool[Literal[True]]
    unbalancedGroup: Bool[Literal[True]]
    hidden: Bool[Literal[False]]
    fieldsUsage: NestedSequence[list[FieldUsage]]
    groupLevels: NestedSequence[list[GroupLevel]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        uniqueName: str = "",
        caption: str | None = None,
        measure: _ConvertibleToBool = None,
        set: _ConvertibleToBool = None,
        parentSet: ConvertibleToInt | None = None,
        iconSet: ConvertibleToInt = 0,
        attribute: _ConvertibleToBool = None,
        time: _ConvertibleToBool = None,
        keyAttribute: _ConvertibleToBool = None,
        defaultMemberUniqueName: str | None = None,
        allUniqueName: str | None = None,
        allCaption: str | None = None,
        dimensionUniqueName: str | None = None,
        displayFolder: str | None = None,
        measureGroup: str | None = None,
        measures: _ConvertibleToBool = None,
        *,
        count: ConvertibleToInt,
        oneField: _ConvertibleToBool = None,
        memberValueDatatype: ConvertibleToInt | None = None,
        unbalanced: _ConvertibleToBool | None = None,
        unbalancedGroup: _ConvertibleToBool | None = None,
        hidden: _ConvertibleToBool = None,
        fieldsUsage: list[FieldUsage] | tuple[FieldUsage, ...] = (),
        groupLevels: list[FieldUsage] | tuple[FieldUsage, ...] = (),
        extLst: ExtensionList | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        uniqueName: str,
        caption: str | None,
        measure: _ConvertibleToBool,
        set: _ConvertibleToBool,
        parentSet: ConvertibleToInt | None,
        iconSet: ConvertibleToInt,
        attribute: _ConvertibleToBool,
        time: _ConvertibleToBool,
        keyAttribute: _ConvertibleToBool,
        defaultMemberUniqueName: str | None,
        allUniqueName: str | None,
        allCaption: str | None,
        dimensionUniqueName: str | None,
        displayFolder: str | None,
        measureGroup: str | None,
        measures: _ConvertibleToBool,
        count: ConvertibleToInt,
        oneField: _ConvertibleToBool = None,
        memberValueDatatype: ConvertibleToInt | None = None,
        unbalanced: _ConvertibleToBool | None = None,
        unbalancedGroup: _ConvertibleToBool | None = None,
        hidden: _ConvertibleToBool = None,
        fieldsUsage: list[FieldUsage] | tuple[FieldUsage, ...] = (),
        groupLevels: list[FieldUsage] | tuple[FieldUsage, ...] = (),
        extLst: ExtensionList | None = None,
    ) -> None: ...

class GroupItems(Serialisable):
    tagname: ClassVar[str]
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(self, count=None, m=(), n=(), b=(), e=(), s=(), d=()) -> None: ...
    @property
    def count(self) -> int: ...

class RangePr(Serialisable):
    tagname: ClassVar[str]
    autoStart: Bool[Literal[True]]
    autoEnd: Bool[Literal[True]]
    groupBy: NoneSet[_RangePrGroupBy]
    startNum: Float[Literal[True]]
    endNum: Float[Literal[True]]
    startDate: DateTime[Literal[True]]
    endDate: DateTime[Literal[True]]
    groupInterval: Float[Literal[True]]
    def __init__(
        self,
        autoStart: _ConvertibleToBool | None = True,
        autoEnd: _ConvertibleToBool | None = True,
        groupBy: _RangePrGroupBy = "range",
        startNum: ConvertibleToFloat | None = None,
        endNum: ConvertibleToFloat | None = None,
        startDate: datetime | str | None = None,
        endDate: datetime | str | None = None,
        groupInterval: ConvertibleToFloat | None = 1,
    ) -> None: ...

class FieldGroup(Serialisable):
    tagname: ClassVar[str]
    par: Integer[Literal[True]]
    base: Integer[Literal[True]]
    rangePr: Typed[RangePr, Literal[True]]
    discretePr: NestedSequence[list[NestedInteger[Literal[False]]]]
    groupItems: Typed[GroupItems, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        par: ConvertibleToInt | None = None,
        base: ConvertibleToInt | None = None,
        rangePr: RangePr | None = None,
        discretePr: list[NestedInteger[Literal[False]]] | tuple[NestedInteger[Literal[False]], ...] = (),
        groupItems: GroupItems | None = None,
    ) -> None: ...

class SharedItems(Serialisable):
    tagname: ClassVar[str]
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    containsSemiMixedTypes: Bool[Literal[True]]
    containsNonDate: Bool[Literal[True]]
    containsDate: Bool[Literal[True]]
    containsString: Bool[Literal[True]]
    containsBlank: Bool[Literal[True]]
    containsMixedTypes: Bool[Literal[True]]
    containsNumber: Bool[Literal[True]]
    containsInteger: Bool[Literal[True]]
    minValue: Float[Literal[True]]
    maxValue: Float[Literal[True]]
    minDate: DateTime[Literal[True]]
    maxDate: DateTime[Literal[True]]
    longText: Bool[Literal[True]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        _fields=(),
        containsSemiMixedTypes: _ConvertibleToBool | None = None,
        containsNonDate: _ConvertibleToBool | None = None,
        containsDate: _ConvertibleToBool | None = None,
        containsString: _ConvertibleToBool | None = None,
        containsBlank: _ConvertibleToBool | None = None,
        containsMixedTypes: _ConvertibleToBool | None = None,
        containsNumber: _ConvertibleToBool | None = None,
        containsInteger: _ConvertibleToBool | None = None,
        minValue: ConvertibleToFloat | None = None,
        maxValue: ConvertibleToFloat | None = None,
        minDate: datetime | str | None = None,
        maxDate: datetime | str | None = None,
        count: Unused = None,
        longText: _ConvertibleToBool | None = None,
    ) -> None: ...
    @property
    def count(self) -> int: ...

class CacheField(Serialisable):
    tagname: ClassVar[str]
    sharedItems: Typed[SharedItems, Literal[True]]
    fieldGroup: Typed[FieldGroup, Literal[True]]
    mpMap: NestedInteger[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    name: String[Literal[False]]
    caption: String[Literal[True]]
    propertyName: String[Literal[True]]
    serverField: Bool[Literal[True]]
    uniqueList: Bool[Literal[True]]
    numFmtId: Integer[Literal[True]]
    formula: String[Literal[True]]
    sqlType: Integer[Literal[True]]
    hierarchy: Integer[Literal[True]]
    level: Integer[Literal[True]]
    databaseField: Bool[Literal[True]]
    mappingCount: Integer[Literal[True]]
    memberPropertyField: Bool[Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        sharedItems: SharedItems | None = None,
        fieldGroup: FieldGroup | None = None,
        mpMap: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        extLst: ExtensionList | None = None,
        *,
        name: str,
        caption: str | None = None,
        propertyName: str | None = None,
        serverField: _ConvertibleToBool | None = None,
        uniqueList: _ConvertibleToBool | None = True,
        numFmtId: ConvertibleToInt | None = None,
        formula: str | None = None,
        sqlType: ConvertibleToInt | None = 0,
        hierarchy: ConvertibleToInt | None = 0,
        level: ConvertibleToInt | None = 0,
        databaseField: _ConvertibleToBool | None = True,
        mappingCount: ConvertibleToInt | None = None,
        memberPropertyField: _ConvertibleToBool | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        sharedItems: SharedItems | None,
        fieldGroup: FieldGroup | None,
        mpMap: Incomplete | None,
        extLst: ExtensionList | None,
        name: str,
        caption: str | None = None,
        propertyName: str | None = None,
        serverField: _ConvertibleToBool | None = None,
        uniqueList: _ConvertibleToBool | None = True,
        numFmtId: ConvertibleToInt | None = None,
        formula: str | None = None,
        sqlType: ConvertibleToInt | None = 0,
        hierarchy: ConvertibleToInt | None = 0,
        level: ConvertibleToInt | None = 0,
        databaseField: _ConvertibleToBool | None = True,
        mappingCount: ConvertibleToInt | None = None,
        memberPropertyField: _ConvertibleToBool | None = None,
    ) -> None: ...

class RangeSet(Serialisable):
    tagname: ClassVar[str]
    i1: Integer[Literal[True]]
    i2: Integer[Literal[True]]
    i3: Integer[Literal[True]]
    i4: Integer[Literal[True]]
    ref: String[Literal[False]]
    name: String[Literal[True]]
    sheet: String[Literal[True]]
    @overload
    def __init__(
        self,
        i1: ConvertibleToInt | None = None,
        i2: ConvertibleToInt | None = None,
        i3: ConvertibleToInt | None = None,
        i4: ConvertibleToInt | None = None,
        *,
        ref: str,
        name: str | None = None,
        sheet: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        i1: ConvertibleToInt | None,
        i2: ConvertibleToInt | None,
        i3: ConvertibleToInt | None,
        i4: ConvertibleToInt | None,
        ref: str,
        name: str | None = None,
        sheet: str | None = None,
    ) -> None: ...

class PageItem(Serialisable):
    tagname: ClassVar[str]
    name: String[Literal[False]]
    def __init__(self, name: str) -> None: ...

class Consolidation(Serialisable):
    tagname: ClassVar[str]
    autoPage: Bool[Literal[True]]
    pages: NestedSequence[list[PageItem]]
    rangeSets: NestedSequence[list[RangeSet]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        autoPage: _ConvertibleToBool | None = None,
        pages: list[PageItem] | tuple[PageItem, ...] = (),
        rangeSets: list[RangeSet] | tuple[RangeSet, ...] = (),
    ) -> None: ...

class WorksheetSource(Serialisable):
    tagname: ClassVar[str]
    ref: String[Literal[True]]
    name: String[Literal[True]]
    sheet: String[Literal[True]]
    def __init__(self, ref: str | None = None, name: str | None = None, sheet: str | None = None) -> None: ...

class CacheSource(Serialisable):
    tagname: ClassVar[str]
    type: Set[_CacheSourceType]
    connectionId: Integer[Literal[True]]
    worksheetSource: Typed[WorksheetSource, Literal[True]]
    consolidation: Typed[Consolidation, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        type: _CacheSourceType,
        connectionId: ConvertibleToInt | None = None,
        worksheetSource: WorksheetSource | None = None,
        consolidation: Consolidation | None = None,
        extLst: ExtensionList | None = None,
    ) -> None: ...

class CacheDefinition(Serialisable):
    mime_type: str
    rel_type: str
    records: Incomplete
    tagname: ClassVar[str]
    invalid: Bool[Literal[True]]
    saveData: Bool[Literal[True]]
    refreshOnLoad: Bool[Literal[True]]
    optimizeMemory: Bool[Literal[True]]
    enableRefresh: Bool[Literal[True]]
    refreshedBy: String[Literal[True]]
    refreshedDate: Float[Literal[True]]
    refreshedDateIso: DateTime[Literal[True]]
    backgroundQuery: Bool[Literal[True]]
    missingItemsLimit: Integer[Literal[True]]
    createdVersion: Integer[Literal[True]]
    refreshedVersion: Integer[Literal[True]]
    minRefreshableVersion: Integer[Literal[True]]
    recordCount: Integer[Literal[True]]
    upgradeOnRefresh: Bool[Literal[True]]
    supportSubquery: Bool[Literal[True]]
    supportAdvancedDrill: Bool[Literal[True]]
    cacheSource: Typed[CacheSource, Literal[True]]
    cacheFields: Incomplete
    cacheHierarchies: Incomplete
    kpis: NestedSequence[list[OLAPKPI]]
    tupleCache: Typed[TupleCache, Literal[True]]
    calculatedItems: Incomplete
    calculatedMembers: Incomplete
    dimensions: Incomplete
    measureGroups: Incomplete
    maps: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    id: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        invalid: _ConvertibleToBool | None = None,
        saveData: _ConvertibleToBool | None = None,
        refreshOnLoad: _ConvertibleToBool | None = None,
        optimizeMemory: _ConvertibleToBool | None = None,
        enableRefresh: _ConvertibleToBool | None = None,
        refreshedBy: str | None = None,
        refreshedDate: ConvertibleToFloat | None = None,
        refreshedDateIso: datetime | str | None = None,
        backgroundQuery: _ConvertibleToBool | None = None,
        missingItemsLimit: ConvertibleToInt | None = None,
        createdVersion: ConvertibleToInt | None = None,
        refreshedVersion: ConvertibleToInt | None = None,
        minRefreshableVersion: ConvertibleToInt | None = None,
        recordCount: ConvertibleToInt | None = None,
        upgradeOnRefresh: _ConvertibleToBool | None = None,
        tupleCache: TupleCache | None = None,
        supportSubquery: _ConvertibleToBool | None = None,
        supportAdvancedDrill: _ConvertibleToBool | None = None,
        *,
        cacheSource: CacheSource,
        cacheFields=(),
        cacheHierarchies=(),
        kpis: list[OLAPKPI] | tuple[OLAPKPI, ...] = (),
        calculatedItems=(),
        calculatedMembers=(),
        dimensions=(),
        measureGroups=(),
        maps=(),
        extLst: ExtensionList | None = None,
        id=None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        invalid: _ConvertibleToBool | None,
        saveData: _ConvertibleToBool | None,
        refreshOnLoad: _ConvertibleToBool | None,
        optimizeMemory: _ConvertibleToBool | None,
        enableRefresh: _ConvertibleToBool | None,
        refreshedBy: str | None,
        refreshedDate: ConvertibleToFloat | None,
        refreshedDateIso: datetime | str | None,
        backgroundQuery: _ConvertibleToBool | None,
        missingItemsLimit: ConvertibleToInt | None,
        createdVersion: ConvertibleToInt | None,
        refreshedVersion: ConvertibleToInt | None,
        minRefreshableVersion: ConvertibleToInt | None,
        recordCount: ConvertibleToInt | None,
        upgradeOnRefresh: _ConvertibleToBool | None,
        tupleCache: TupleCache | None,
        supportSubquery: _ConvertibleToBool | None,
        supportAdvancedDrill: _ConvertibleToBool | None,
        cacheSource: CacheSource,
        cacheFields=(),
        cacheHierarchies=(),
        kpis=(),
        calculatedItems=(),
        calculatedMembers=(),
        dimensions=(),
        measureGroups=(),
        maps=(),
        extLst: ExtensionList | None = None,
        id=None,
    ) -> None: ...
    def to_tree(self) -> Element: ...  # type: ignore[override]
    @property
    def path(self) -> str: ...



from typing import overload

from pymeeus.Angle import Angle

class Interpolation:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, a: Interpolation, /) -> None: ...
    @overload
    def __init__(self, a: list[float | Angle] | tuple[float | Angle, ...], /) -> None: ...
    @overload
    def __init__(
        self, a1: list[float | Angle] | tuple[float | Angle, ...], a2: list[float | Angle] | tuple[float | Angle, ...], /
    ) -> None: ...
    @overload
    def __init__(
        self, a1: float | Angle, a2: float | Angle, a3: float | Angle, a4: float | Angle, /, *rest: float | Angle
    ) -> None: ...
    @overload
    def set(self) -> None: ...
    @overload
    def set(self, a: Interpolation, /) -> None: ...
    @overload
    def set(self, a: list[float | Angle] | tuple[float | Angle, ...], /) -> None: ...
    @overload
    def set(
        self, a1: list[float | Angle] | tuple[float | Angle, ...], a2: list[float | Angle] | tuple[float | Angle, ...], /
    ) -> None: ...
    @overload
    def set(
        self, a1: float | Angle, a2: float | Angle, a3: float | Angle, a4: float | Angle, /, *rest: float | Angle
    ) -> None: ...
    def __len__(self) -> int: ...
    def get_tolerance(self) -> float: ...
    def set_tolerance(self, tol: float) -> None: ...
    def __call__(self, x: float | Angle) -> float | Angle: ...
    def derivative(self, x: float | Angle) -> float: ...
    def root(self, xl: float | Angle = 0, xh: float | Angle = 0, max_iter: int = 1000) -> float | Angle: ...
    def minmax(self, xl: float | Angle = 0, xh: float | Angle = 0, max_iter: int = 1000) -> float | Angle: ...

def main() -> None: ...



METAFILE: str

def get_current_context_name() -> str: ...
def write_context_name_to_docker_config(name: str | None = None) -> Exception | None: ...
def get_context_id(name: str) -> str: ...
def get_context_dir() -> str: ...
def get_meta_dir(name: str | None = None) -> str: ...
def get_meta_file(name: str) -> str: ...
def get_tls_dir(name: str | None = None, endpoint: str = "") -> str: ...
def get_context_host(path: str | None = None, tls: bool = False) -> str: ...



from typing_extensions import Self

from .context import CryptContext
from .hash import htdigest

class _CommonFile:
    encoding: str
    return_unicode: bool
    autosave: bool
    @classmethod
    def from_string(
        cls,
        data: str | bytes,
        *,
        new: bool = False,
        autoload: bool = True,
        autosave: bool = False,
        encoding: str = "utf-8",
        return_unicode: bool = True,
    ) -> Self: ...
    @classmethod
    def from_path(
        cls,
        path: str,
        *,
        new: bool = False,
        autoload: bool = True,
        autosave: bool = False,
        encoding: str = "utf-8",
        return_unicode: bool = True,
    ) -> Self: ...
    def __init__(
        self,
        path: str | None = None,
        new: bool = False,
        autoload: bool = True,
        autosave: bool = False,
        encoding: str = "utf-8",
        return_unicode: bool = True,
    ) -> None: ...
    @property
    def path(self) -> str: ...
    @path.setter
    def path(self, value: str) -> None: ...
    @property
    def mtime(self) -> float: ...
    def load_if_changed(self) -> bool: ...
    def load(self, path: str | None = None, force: bool = True) -> bool: ...
    def load_string(self, data: str | bytes) -> None: ...
    def save(self, path: str | None = None) -> None: ...
    def to_string(self) -> bytes: ...

class HtpasswdFile(_CommonFile):
    context: CryptContext
    def __init__(
        self,
        path: str | None = None,
        default_scheme: str | None = None,
        context: CryptContext = ...,
        *,
        new: bool = False,
        autoload: bool = True,
        autosave: bool = False,
        encoding: str = "utf-8",
        return_unicode: bool = True,
    ) -> None: ...
    def users(self) -> list[str | bytes]: ...
    def set_password(self, user: str, password: str | bytes) -> bool: ...
    def update(self, user: str, password: str | bytes) -> bool: ...
    def get_hash(self, user: str) -> bytes | None: ...
    def set_hash(self, user: str, hash: str | bytes) -> bool: ...
    def find(self, user: str) -> bytes | None: ...
    def delete(self, user: str) -> bool: ...
    def check_password(self, user: str, password: str | bytes) -> bool | None: ...
    def verify(self, user: str, password: str | bytes) -> bool | None: ...

class HtdigestFile(_CommonFile):
    default_realm: str | None
    def __init__(
        self,
        path: str | None = None,
        default_realm: str | None = None,
        *,
        new: bool = False,
        autoload: bool = True,
        autosave: bool = False,
        encoding: str = "utf-8",
        return_unicode: bool = True,
    ) -> None: ...
    def realms(self) -> list[str | bytes]: ...
    def users(self, realm: str | None = None) -> list[str | bytes]: ...
    def set_password(self, user: str, realm: str | None = None, password: str | bytes = ...) -> bool: ...
    def update(self, user: str, realm: str | None, password: str | bytes) -> bool: ...
    def get_hash(self, user: str, realm: str | None = None) -> htdigest | None: ...
    def set_hash(self, user: str, realm: str | None = None, hash: str | bytes = ...) -> bool: ...
    def find(self, user: str, realm: str | None) -> htdigest | None: ...
    def delete(self, user: str, realm: str | None = None) -> bool: ...
    def delete_realm(self, realm: str | None) -> int: ...
    def check_password(self, user: str, realm: str | None = None, password: str | bytes = ...) -> bool | None: ...
    def verify(self, user: str, realm: str | None, password: str | bytes) -> bool | None: ...

__all__ = ["HtpasswdFile", "HtdigestFile"]



from _typeshed import Incomplete
from collections.abc import Callable, Mapping
from logging import Logger
from typing import Any, Final

from oauthlib.common import _HTTPMethod

log: Logger
SIGNATURE_HMAC_SHA1: Final[str]
SIGNATURE_HMAC_SHA256: Final[str]
SIGNATURE_HMAC_SHA512: Final[str]
SIGNATURE_HMAC: Final[str]
SIGNATURE_RSA_SHA1: Final[str]
SIGNATURE_RSA_SHA256: Final[str]
SIGNATURE_RSA_SHA512: Final[str]
SIGNATURE_RSA: Final[str]
SIGNATURE_PLAINTEXT: Final[str]
SIGNATURE_METHODS: Final[tuple[str, str, str, str, str, str, str]]
SIGNATURE_TYPE_AUTH_HEADER: Final[str]
SIGNATURE_TYPE_QUERY: Final[str]
SIGNATURE_TYPE_BODY: Final[str]
CONTENT_TYPE_FORM_URLENCODED: Final[str]

class Client:
    SIGNATURE_METHODS: dict[str, Callable[[str, Incomplete], str]]
    @classmethod
    def register_signature_method(cls, method_name, method_callback) -> None: ...
    client_key: Any
    client_secret: Any
    resource_owner_key: Any
    resource_owner_secret: Any
    signature_method: Any
    signature_type: Any
    callback_uri: Any
    rsa_key: Any
    verifier: Any
    realm: Any
    encoding: Any
    decoding: Any
    nonce: Any
    timestamp: Any
    def __init__(
        self,
        client_key: str,
        client_secret: str | None = None,
        resource_owner_key=None,
        resource_owner_secret=None,
        callback_uri=None,
        signature_method="HMAC-SHA1",
        signature_type="AUTH_HEADER",
        rsa_key=None,
        verifier=None,
        realm=None,
        encoding: str = "utf-8",
        decoding=None,
        nonce=None,
        timestamp=None,
    ): ...
    def get_oauth_signature(self, request): ...
    def get_oauth_params(self, request): ...
    def sign(
        self,
        uri: str,
        http_method: _HTTPMethod = "GET",
        body: str | dict[str, str] | list[tuple[str, str]] | None = None,
        headers: Mapping[str, str] | None = None,
        realm=None,
    ): ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_distance_regular", "is_strongly_regular", "intersection_array", "global_parameters"]

@_dispatchable
def is_distance_regular(G: Graph[_Node]) -> bool: ...
@_dispatchable
def global_parameters(b, c) -> Generator[tuple[Incomplete, Incomplete, Incomplete]]: ...
@_dispatchable
def intersection_array(G: Graph[_Node]): ...
@_dispatchable
def is_strongly_regular(G: Graph[_Node]) -> bool: ...



from collections.abc import Callable, Iterable, MutableMapping, Sequence
from types import ModuleType
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Self

_T = TypeVar("_T")

WRAPPER_ASSIGNMENTS: tuple[str, ...]
WRAPPER_UPDATES: tuple[str, ...]

def update_wrapper(
    wrapper: _T,
    wrapped: object,
    assigned: Sequence[str] = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__"),
    updated: Sequence[str] = ("__dict__",),
) -> _T: ...
def copy_globals(
    source: ModuleType,
    globs: MutableMapping[str, Any],
    only_names: Iterable[str] | None = None,
    ignore_missing_names: bool = False,
    names_to_ignore: Sequence[str] = (),
    dunder_names_to_keep: Sequence[str] = ("__implements__", "__all__", "__imports__"),
    cleanup_globs: bool = True,
) -> list[str]: ...
def import_c_accel(globs: MutableMapping[str, Any], cname: str) -> None: ...

class Lazy(Generic[_T]):
    data: _T
    def __init__(self, func: Callable[[Any], _T]) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

class readproperty(Generic[_T]):
    func: Callable[[Any], _T]
    def __init__(
        self: readproperty[_T], func: Callable[[Any], _T]  # pyright: ignore[reportInvalidTypeVarUse]  #11780
    ) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

class LazyOnClass(Generic[_T]):
    @classmethod
    def lazy(cls, cls_dict: MutableMapping[str, Any], func: Callable[[Any], _T]) -> None: ...
    name: str
    func: Callable[[Any], _T]
    def __init__(self, func: Callable[[Any], _T], name: str | None = None) -> None: ...
    @overload
    def __get__(self, inst: None, class_: type[object]) -> Self: ...
    @overload
    def __get__(self, inst: object, class_: type[object]) -> _T: ...

def gmctime() -> str: ...
def prereleaser_middle(data: MutableMapping[str, Any]) -> None: ...
def postreleaser_before(data: MutableMapping[str, Any]) -> None: ...



from _typeshed import Unused
from collections.abc import Callable

from oauthlib.common import Request
from oauthlib.oauth2.rfc8628.grant_types import DeviceCodeGrant

from ..grant_types import (
    AuthorizationCodeGrant,
    ClientCredentialsGrant,
    ImplicitGrant,
    RefreshTokenGrant,
    ResourceOwnerPasswordCredentialsGrant,
)
from ..request_validator import RequestValidator
from ..tokens import BearerToken
from .authorization import AuthorizationEndpoint
from .introspect import IntrospectEndpoint
from .resource import ResourceEndpoint
from .revocation import RevocationEndpoint
from .token import TokenEndpoint

class Server(AuthorizationEndpoint, IntrospectEndpoint, TokenEndpoint, ResourceEndpoint, RevocationEndpoint):
    auth_grant: AuthorizationCodeGrant
    implicit_grant: ImplicitGrant
    password_grant: ResourceOwnerPasswordCredentialsGrant
    credentials_grant: ClientCredentialsGrant
    refresh_grant: RefreshTokenGrant
    device_code_grant: DeviceCodeGrant
    bearer: BearerToken
    def __init__(
        self,
        request_validator: RequestValidator,
        token_expires_in: int | Callable[[Request], int] | None = None,
        token_generator: Callable[[Request], str] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
        *args: Unused,
    ) -> None: ...

class WebApplicationServer(AuthorizationEndpoint, IntrospectEndpoint, TokenEndpoint, ResourceEndpoint, RevocationEndpoint):
    auth_grant: AuthorizationCodeGrant
    refresh_grant: RefreshTokenGrant
    bearer: BearerToken
    def __init__(
        self,
        request_validator: RequestValidator,
        token_generator: Callable[[Request], str] | None = None,
        token_expires_in: int | Callable[[Request], int] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
    ) -> None: ...

class MobileApplicationServer(AuthorizationEndpoint, IntrospectEndpoint, ResourceEndpoint, RevocationEndpoint):
    implicit_grant: ImplicitGrant
    bearer: BearerToken
    def __init__(
        self,
        request_validator: RequestValidator,
        token_generator: Callable[[Request], str] | None = None,
        token_expires_in: int | Callable[[Request], int] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
    ) -> None: ...

class LegacyApplicationServer(TokenEndpoint, IntrospectEndpoint, ResourceEndpoint, RevocationEndpoint):
    password_grant: ResourceOwnerPasswordCredentialsGrant
    refresh_grant: RefreshTokenGrant
    bearer: BearerToken
    def __init__(
        self,
        request_validator: RequestValidator,
        token_generator: Callable[[Request], str] | None = None,
        token_expires_in: int | Callable[[Request], int] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
    ) -> None: ...

class BackendApplicationServer(TokenEndpoint, IntrospectEndpoint, ResourceEndpoint, RevocationEndpoint):
    credentials_grant: ClientCredentialsGrant
    bearer: BearerToken
    def __init__(
        self,
        request_validator: RequestValidator,
        token_generator: Callable[[Request], str] | None = None,
        token_expires_in: int | Callable[[Request], int] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
    ) -> None: ...



from typing import Final

class AuthenticationIds:
    ThreeDSecureVisaFullAuthentication: Final = "fake-three-d-secure-visa-full-authentication-id"
    ThreeDSecureVisaLookupTimeout: Final = "fake-three-d-secure-visa-lookup-timeout-id"
    ThreeDSecureVisaFailedSignature: Final = "fake-three-d-secure-visa-failed-signature-id"
    ThreeDSecureVisaFailedAuthentication: Final = "fake-three-d-secure-visa-failed-authentication-id"
    ThreeDSecureVisaAttemptsNonParticipating: Final[str]
    ThreeDSecureVisaNoteEnrolled: Final = "fake-three-d-secure-visa-not-enrolled-id"
    ThreeDSecureVisaUnavailable: Final = "fake-three-d-secure-visa-unavailable-id"
    ThreeDSecureVisaMPILookupError: Final = "fake-three-d-secure-visa-mpi-lookup-error-id"
    ThreeDSecureVisaMPIAuthenticateError: Final = "fake-three-d-secure-visa-mpi-authenticate-error-id"
    ThreeDSecureVisaAuthenticationUnavailable: Final[str]
    ThreeDSecureVisaBypassedAuthentication: Final[str]
    ThreeDSecureTwoVisaSuccessfulFrictionlessAuthentication: Final[str]
    ThreeDSecureTwoVisaSuccessfulStepUpAuthentication: Final[str]
    ThreeDSecureTwoVisaErrorOnLookup: Final = "fake-three-d-secure-two-visa-error-on-lookup-id"
    ThreeDSecureTwoVisaTimeoutOnLookup: Final = "fake-three-d-secure-two-visa-timeout-on-lookup-id"



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Actions:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get_actions(
        self,
        trigger_id: str | None = None,
        action_name: str | None = None,
        deployed: bool | None = None,
        installed: bool = False,
        page: int | None = None,
        per_page: int | None = None,
    ): ...
    async def get_actions_async(
        self,
        trigger_id: str | None = None,
        action_name: str | None = None,
        deployed: bool | None = None,
        installed: bool = False,
        page: int | None = None,
        per_page: int | None = None,
    ): ...
    def create_action(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_action_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def update_action(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_action_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_action(self, id: str) -> dict[str, Incomplete]: ...
    async def get_action_async(self, id: str) -> dict[str, Incomplete]: ...
    def delete_action(self, id: str, force: bool = False): ...
    async def delete_action_async(self, id: str, force: bool = False): ...
    def get_triggers(self) -> dict[str, Incomplete]: ...
    async def get_triggers_async(self) -> dict[str, Incomplete]: ...
    def get_execution(self, id: str) -> dict[str, Incomplete]: ...
    async def get_execution_async(self, id: str) -> dict[str, Incomplete]: ...
    def get_action_versions(self, id: str, page: int | None = None, per_page: int | None = None) -> dict[str, Incomplete]: ...
    async def get_action_versions_async(
        self, id: str, page: int | None = None, per_page: int | None = None
    ) -> dict[str, Incomplete]: ...
    def get_trigger_bindings(self, id: str, page: int | None = None, per_page: int | None = None) -> dict[str, Incomplete]: ...
    async def get_trigger_bindings_async(
        self, id: str, page: int | None = None, per_page: int | None = None
    ) -> dict[str, Incomplete]: ...
    def get_action_version(self, action_id: str, version_id: str) -> dict[str, Incomplete]: ...
    async def get_action_version_async(self, action_id: str, version_id: str) -> dict[str, Incomplete]: ...
    def deploy_action(self, id: str) -> dict[str, Incomplete]: ...
    async def deploy_action_async(self, id: str) -> dict[str, Incomplete]: ...
    def rollback_action_version(self, action_id: str, version_id: str) -> dict[str, Incomplete]: ...
    async def rollback_action_version_async(self, action_id: str, version_id: str) -> dict[str, Incomplete]: ...
    def update_trigger_bindings(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_trigger_bindings_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...



from collections.abc import Callable, Iterable as _Iterable, Mapping
from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class ExtractingMixin:
    def extracting(
        self,
        *names: str,
        filter: str | Mapping[str, Any] | Callable[[Any], bool] = ...,
        sort: str | _Iterable[str] | Callable[[Any], Any] = ...,
    ) -> Self: ...



from braintree.environment import Environment

class CredentialsParser:
    client_id: str | None
    client_secret: str | None
    access_token: str | None
    environment: Environment | None
    merchant_id: str
    def __init__(
        self, client_id: str | None = None, client_secret: str | None = None, access_token: str | None = None
    ) -> None: ...
    def parse_client_credentials(self) -> None: ...
    def parse_access_token(self) -> None: ...
    def get_environment(self, credential: str) -> Environment | None: ...
    def get_merchant_id(self, credential: str) -> str: ...



from _typeshed import Incomplete
from collections.abc import Callable, Generator

from ..utils import ObjectWrapper

__all__ = ["tenumerate", "tzip", "tmap"]

class DummyTqdmFile(ObjectWrapper):
    def __init__(self, wrapped) -> None: ...
    def write(self, x, nolock: bool = False) -> None: ...
    def __del__(self) -> None: ...

def tenumerate(iterable, start: int = 0, total=None, tqdm_class: type[Incomplete] = ..., **tqdm_kwargs): ...
def tzip(iter1, *iter2plus, **tqdm_kwargs) -> Generator[Incomplete, None, None]: ...
def tmap(function: Callable[..., Incomplete], *sequences, **tqdm_kwargs) -> Generator[Incomplete, None, None]: ...



import re
from codecs import CodecInfo
from collections.abc import Generator, Iterable, Iterator
from typing import Any, Final, Protocol, type_check_only

# We don't re-export any `html5lib` types / values here, because they are not
# really public and may change at any time. This is just a helper module,
# import things directly from `html5lib` instead!
from html5lib import HTMLParser
from html5lib._inputstream import HTMLBinaryInputStream, HTMLUnicodeInputStream
from html5lib._tokenizer import HTMLTokenizer
from html5lib._trie import Trie
from html5lib.serializer import HTMLSerializer
from html5lib.treewalkers.base import TreeWalker

# Is actually webencodings.Encoding
@type_check_only
class _Encoding(Protocol):
    name: str
    codec_info: CodecInfo
    def __init__(self, name: str, codec_info: CodecInfo) -> None: ...

HTML_TAGS: Final[frozenset[str]]
HTML_TAGS_BLOCK_LEVEL: Final[frozenset[str]]
AMP_SPLIT_RE: Final[re.Pattern[str]]
ENTITIES: Final[dict[str, str]]
ENTITIES_TRIE: Final[Trie]
TAG_TOKEN_TYPES: Final[set[int]]
TAG_TOKEN_TYPE_CHARACTERS: Final[int]
TAG_TOKEN_TYPE_END: Final[int]
TAG_TOKEN_TYPE_PARSEERROR: Final[int]
TAG_TOKEN_TYPE_START: Final[int]

class InputStreamWithMemory:
    position = HTMLUnicodeInputStream.position
    reset = HTMLUnicodeInputStream.reset
    def __init__(self, inner_stream: HTMLUnicodeInputStream) -> None: ...
    @property
    def errors(self) -> list[str]: ...
    @property
    def charEncoding(self) -> tuple[_Encoding, str]: ...
    # If inner_stream wasn't a HTMLBinaryInputStream, this will error at runtime
    # Is a property returning a method, simplified:
    changeEncoding = HTMLBinaryInputStream.changeEncoding
    def char(self) -> str: ...
    def charsUntil(self, characters: Iterable[str], opposite: bool = False) -> str: ...
    def unget(self, char: str | None) -> None: ...
    def get_tag(self) -> str: ...
    def start_tag(self) -> None: ...

class BleachHTMLTokenizer(HTMLTokenizer):
    consume_entities: bool
    stream: InputStreamWithMemory  # type: ignore[assignment]
    emitted_last_token: dict[str, Any] | None
    def __init__(self, consume_entities: bool = False, **kwargs: Any) -> None: ...

class BleachHTMLParser(HTMLParser):
    tags: list[str] | None
    strip: bool
    consume_entities: bool
    def __init__(self, tags: Iterable[str] | None, strip: bool, consume_entities: bool, **kwargs: Any) -> None: ...

class BleachHTMLSerializer(HTMLSerializer):
    escape_rcdata: bool
    def escape_base_amp(self, stoken: str) -> Generator[str, None, None]: ...
    def serialize(self, treewalker: TreeWalker, encoding: str | None = None) -> Generator[str, None, None]: ...  # type: ignore[override]

def convert_entity(value: str) -> str | None: ...
def convert_entities(text: str) -> str: ...
def match_entity(stream: str) -> str | None: ...
def next_possible_entity(text: str) -> Iterator[str]: ...



import sys
from html import escape as escape
from io import FileIO, TextIOWrapper
from queue import Empty as Empty, Queue as Queue
from typing import IO

if sys.version_info >= (3, 13):
    # NOTE: These are the only attributes we realistically care about
    class cgi_FieldStorage:
        filename: str
        file: IO[bytes]
        def make_file(self) -> TextIOWrapper | FileIO: ...

    def parse_header(line: str) -> tuple[str, dict[str, str]]: ...

else:
    from cgi import FieldStorage as _cgi_FieldStorage, parse_header as parse_header

    class cgi_FieldStorage(_cgi_FieldStorage):
        # NOTE: The only kinds of objects of this type the user is exposed to
        #       will contain a file with a filename. We're technically lying
        #       if people create their own instances, but that shouldn't happen
        filename: str
        file: IO[bytes]



from _typeshed import Incomplete, StrPath
from types import TracebackType
from typing import Any
from typing_extensions import Self

from ..dist import Distribution
from . import expand

def load_file(filepath: StrPath) -> dict[Incomplete, Incomplete]: ...
def validate(config: dict[Incomplete, Incomplete], filepath: StrPath) -> bool: ...
def apply_configuration(dist: Distribution, filepath: StrPath, ignore_option_errors: bool = False) -> Distribution: ...
def read_configuration(
    filepath: StrPath, expand: bool = True, ignore_option_errors: bool = False, dist: Distribution | None = None
) -> dict[str, Any]: ...
def expand_configuration(
    config: dict[Incomplete, Incomplete],
    root_dir: StrPath | None = None,
    ignore_option_errors: bool = False,
    dist: Distribution | None = None,
) -> dict[Incomplete, Incomplete]: ...

class _ConfigExpander:
    config: dict[Incomplete, Incomplete]
    root_dir: StrPath
    project_cfg: Incomplete
    dynamic: Incomplete
    setuptools_cfg: Incomplete
    dynamic_cfg: Incomplete
    ignore_option_errors: bool
    def __init__(
        self,
        config: dict[Incomplete, Incomplete],
        root_dir: StrPath | None = None,
        ignore_option_errors: bool = False,
        dist: Distribution | None = None,
    ) -> None: ...
    def expand(self): ...

class _EnsurePackagesDiscovered(expand.EnsurePackagesDiscovered):
    def __init__(
        self, distribution: Distribution, project_cfg: dict[Incomplete, Incomplete], setuptools_cfg: dict[Incomplete, Incomplete]
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...



############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from functools import reduce
from typing import Any, Dict, List, Optional, Set, Union, TypeVar
import logging

from lsprotocol import types


logger = logging.getLogger(__name__)
T = TypeVar("T")


def get_capability(
    client_capabilities: types.ClientCapabilities, field: str, default: Any = None
) -> Any:
    """Check if ClientCapabilities has some nested value without raising
    AttributeError.
    e.g. get_capability('text_document.synchronization.will_save')
    """
    try:
        value = reduce(getattr, field.split("."), client_capabilities)
    except AttributeError:
        return default

    # If we reach the desired leaf value but it's None, return the default.
    return default if value is None else value


class ServerCapabilitiesBuilder:
    """Create `ServerCapabilities` instance depending on builtin and user registered
    features.
    """

    def __init__(
        self,
        client_capabilities: types.ClientCapabilities,
        features: Set[str],
        feature_options: Dict[str, Any],
        commands: List[str],
        text_document_sync_kind: types.TextDocumentSyncKind,
        notebook_document_sync: Optional[types.NotebookDocumentSyncOptions] = None,
    ):
        self.client_capabilities = client_capabilities
        self.features = features
        self.feature_options = feature_options
        self.commands = commands
        self.text_document_sync_kind = text_document_sync_kind
        self.notebook_document_sync = notebook_document_sync

        self.server_cap = types.ServerCapabilities()

    def _provider_options(self, feature: str, default: T) -> Optional[Union[T, Any]]:
        if feature in self.features:
            return self.feature_options.get(feature, default)
        return None

    def _with_text_document_sync(self):
        open_close = (
            types.TEXT_DOCUMENT_DID_OPEN in self.features
            or types.TEXT_DOCUMENT_DID_CLOSE in self.features
        )
        will_save = (
            get_capability(
                self.client_capabilities, "text_document.synchronization.will_save"
            )
            and types.TEXT_DOCUMENT_WILL_SAVE in self.features
        )
        will_save_wait_until = (
            get_capability(
                self.client_capabilities,
                "text_document.synchronization.will_save_wait_until",
            )
            and types.TEXT_DOCUMENT_WILL_SAVE_WAIT_UNTIL in self.features
        )
        if types.TEXT_DOCUMENT_DID_SAVE in self.features:
            save = self.feature_options.get(types.TEXT_DOCUMENT_DID_SAVE, True)
        else:
            save = False

        self.server_cap.text_document_sync = types.TextDocumentSyncOptions(
            open_close=open_close,
            change=self.text_document_sync_kind,
            will_save=will_save,
            will_save_wait_until=will_save_wait_until,
            save=save,
        )

        return self

    def _with_notebook_document_sync(self):
        if self.client_capabilities.notebook_document is None:
            return self

        self.server_cap.notebook_document_sync = self.notebook_document_sync
        return self

    def _with_completion(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_COMPLETION, default=types.CompletionOptions()
        )
        if value is not None:
            self.server_cap.completion_provider = value
        return self

    def _with_hover(self):
        value = self._provider_options(types.TEXT_DOCUMENT_HOVER, default=True)
        if value is not None:
            self.server_cap.hover_provider = value
        return self

    def _with_signature_help(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_SIGNATURE_HELP, default=types.SignatureHelpOptions()
        )
        if value is not None:
            self.server_cap.signature_help_provider = value
        return self

    def _with_declaration(self):
        value = self._provider_options(types.TEXT_DOCUMENT_DECLARATION, default=True)
        if value is not None:
            self.server_cap.declaration_provider = value
        return self

    def _with_definition(self):
        value = self._provider_options(types.TEXT_DOCUMENT_DEFINITION, default=True)
        if value is not None:
            self.server_cap.definition_provider = value
        return self

    def _with_type_definition(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_TYPE_DEFINITION, default=types.TypeDefinitionOptions()
        )
        if value is not None:
            self.server_cap.type_definition_provider = value
        return self

    def _with_inlay_hints(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_INLAY_HINT, default=types.InlayHintOptions()
        )
        if value is not None:
            value.resolve_provider = types.INLAY_HINT_RESOLVE in self.features
            self.server_cap.inlay_hint_provider = value
        return self

    def _with_implementation(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_IMPLEMENTATION, default=types.ImplementationOptions()
        )
        if value is not None:
            self.server_cap.implementation_provider = value
        return self

    def _with_references(self):
        value = self._provider_options(types.TEXT_DOCUMENT_REFERENCES, default=True)
        if value is not None:
            self.server_cap.references_provider = value
        return self

    def _with_document_highlight(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_DOCUMENT_HIGHLIGHT, default=True
        )
        if value is not None:
            self.server_cap.document_highlight_provider = value
        return self

    def _with_document_symbol(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_DOCUMENT_SYMBOL, default=True
        )
        if value is not None:
            self.server_cap.document_symbol_provider = value
        return self

    def _with_code_action(self):
        value = self._provider_options(types.TEXT_DOCUMENT_CODE_ACTION, default=True)
        if value is not None:
            self.server_cap.code_action_provider = value
        return self

    def _with_code_lens(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_CODE_LENS, default=types.CodeLensOptions()
        )
        if value is not None:
            self.server_cap.code_lens_provider = value
        return self

    def _with_document_link(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_DOCUMENT_LINK, default=types.DocumentLinkOptions()
        )
        if value is not None:
            self.server_cap.document_link_provider = value
        return self

    def _with_color(self):
        value = self._provider_options(types.TEXT_DOCUMENT_DOCUMENT_COLOR, default=True)
        if value is not None:
            self.server_cap.color_provider = value
        return self

    def _with_document_formatting(self):
        value = self._provider_options(types.TEXT_DOCUMENT_FORMATTING, default=True)
        if value is not None:
            self.server_cap.document_formatting_provider = value
        return self

    def _with_document_range_formatting(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_RANGE_FORMATTING, default=True
        )
        if value is not None:
            self.server_cap.document_range_formatting_provider = value
        return self

    def _with_document_on_type_formatting(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_ON_TYPE_FORMATTING, default=None
        )
        if value is not None:
            self.server_cap.document_on_type_formatting_provider = value
        return self

    def _with_rename(self):
        value = self._provider_options(types.TEXT_DOCUMENT_RENAME, default=True)
        if value is not None:
            self.server_cap.rename_provider = value
        return self

    def _with_folding_range(self):
        value = self._provider_options(types.TEXT_DOCUMENT_FOLDING_RANGE, default=True)
        if value is not None:
            self.server_cap.folding_range_provider = value
        return self

    def _with_execute_command(self):
        self.server_cap.execute_command_provider = types.ExecuteCommandOptions(
            commands=self.commands
        )
        return self

    def _with_selection_range(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_SELECTION_RANGE, default=True
        )
        if value is not None:
            self.server_cap.selection_range_provider = value
        return self

    def _with_call_hierarchy(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_PREPARE_CALL_HIERARCHY, default=True
        )
        if value is not None:
            self.server_cap.call_hierarchy_provider = value
        return self

    def _with_type_hierarchy(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_PREPARE_TYPE_HIERARCHY, default=True
        )
        if value is not None:
            self.server_cap.type_hierarchy_provider = value
        return self

    def _with_semantic_tokens(self):
        providers = [
            types.TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL,
            types.TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL_DELTA,
            types.TEXT_DOCUMENT_SEMANTIC_TOKENS_RANGE,
        ]

        value = None
        for provider in providers:
            value = self._provider_options(provider, default=None)
            if value is not None:
                break

        if value is None:
            return self

        if isinstance(value, types.SemanticTokensRegistrationOptions):
            self.server_cap.semantic_tokens_provider = value
            return self

        full_support: Union[bool, types.SemanticTokensOptionsFullType1] = (
            types.TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL in self.features
        )

        if types.TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL_DELTA in self.features:
            full_support = types.SemanticTokensOptionsFullType1(delta=True)

        options = types.SemanticTokensOptions(
            legend=value,
            full=full_support or None,
            range=types.TEXT_DOCUMENT_SEMANTIC_TOKENS_RANGE in self.features or None,
        )

        if options.full or options.range:
            self.server_cap.semantic_tokens_provider = options

        return self

    def _with_linked_editing_range(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_LINKED_EDITING_RANGE, default=True
        )
        if value is not None:
            self.server_cap.linked_editing_range_provider = value
        return self

    def _with_moniker(self):
        value = self._provider_options(types.TEXT_DOCUMENT_MONIKER, default=True)
        if value is not None:
            self.server_cap.moniker_provider = value
        return self

    def _with_workspace_symbol(self):
        value = self._provider_options(
            types.WORKSPACE_SYMBOL, default=types.WorkspaceSymbolOptions()
        )
        if value is not None:
            value.resolve_provider = types.WORKSPACE_SYMBOL_RESOLVE in self.features
            self.server_cap.workspace_symbol_provider = value
        return self

    def _with_workspace_capabilities(self):
        # File operations
        file_operations = types.FileOperationOptions()
        operations = [
            (types.WORKSPACE_WILL_CREATE_FILES, "will_create"),
            (types.WORKSPACE_DID_CREATE_FILES, "did_create"),
            (types.WORKSPACE_WILL_DELETE_FILES, "will_delete"),
            (types.WORKSPACE_DID_DELETE_FILES, "did_delete"),
            (types.WORKSPACE_WILL_RENAME_FILES, "will_rename"),
            (types.WORKSPACE_DID_RENAME_FILES, "did_rename"),
        ]

        for method_name, capability_name in operations:
            client_supports_method = get_capability(
                self.client_capabilities, f"workspace.file_operations.{capability_name}"
            )

            if client_supports_method:
                value = self._provider_options(method_name, default=None)
                setattr(file_operations, capability_name, value)

        self.server_cap.workspace = types.ServerCapabilitiesWorkspaceType(
            workspace_folders=types.WorkspaceFoldersServerCapabilities(
                supported=True,
                change_notifications=True,
            ),
            file_operations=file_operations,
        )
        return self

    def _with_diagnostic_provider(self):
        value = self._provider_options(
            types.TEXT_DOCUMENT_DIAGNOSTIC,
            default=types.DiagnosticOptions(
                inter_file_dependencies=False, workspace_diagnostics=False
            ),
        )
        if value is not None:
            value.workspace_diagnostics = types.WORKSPACE_DIAGNOSTIC in self.features
            self.server_cap.diagnostic_provider = value
        return self

    def _with_inline_value_provider(self):
        value = self._provider_options(types.TEXT_DOCUMENT_INLINE_VALUE, default=True)
        if value is not None:
            self.server_cap.inline_value_provider = value
        return self

    def _with_position_encodings(self):
        self.server_cap.position_encoding = types.PositionEncodingKind.Utf16

        general = self.client_capabilities.general
        if general is None:
            return self

        encodings = general.position_encodings
        if encodings is None:
            return self

        if types.PositionEncodingKind.Utf16 in encodings:
            return self

        if types.PositionEncodingKind.Utf32 in encodings:
            self.server_cap.position_encoding = types.PositionEncodingKind.Utf32
            return self

        if types.PositionEncodingKind.Utf8 in encodings:
            self.server_cap.position_encoding = types.PositionEncodingKind.Utf8
            return self

        logger.warning(f"Unknown `PositionEncoding`s: {encodings}")

        return self

    def _build(self):
        return self.server_cap

    def build(self):
        return (
            self._with_text_document_sync()
            ._with_notebook_document_sync()
            ._with_completion()
            ._with_hover()
            ._with_signature_help()
            ._with_declaration()
            ._with_definition()
            ._with_type_definition()
            ._with_inlay_hints()
            ._with_implementation()
            ._with_references()
            ._with_document_highlight()
            ._with_document_symbol()
            ._with_code_action()
            ._with_code_lens()
            ._with_document_link()
            ._with_color()
            ._with_document_formatting()
            ._with_document_range_formatting()
            ._with_document_on_type_formatting()
            ._with_rename()
            ._with_folding_range()
            ._with_execute_command()
            ._with_selection_range()
            ._with_call_hierarchy()
            ._with_type_hierarchy()
            ._with_semantic_tokens()
            ._with_linked_editing_range()
            ._with_moniker()
            ._with_workspace_symbol()
            ._with_workspace_capabilities()
            ._with_diagnostic_provider()
            ._with_inline_value_provider()
            ._with_position_encodings()
            ._build()
        )



from typing import Literal, TypedDict, type_check_only

from .reservoir import Reservoir

@type_check_only
class _Stats(TypedDict):
    request_count: int
    borrow_count: int
    sampled_count: int

class SamplingRule:
    def __init__(
        self, name: str, priority, rate, reservoir_size, host=None, method=None, path=None, service=None, service_type=None
    ) -> None: ...
    def match(self, sampling_req) -> bool: ...
    def is_default(self) -> bool: ...
    def snapshot_statistics(self) -> _Stats: ...
    def merge(self, rule) -> None: ...
    def ever_matched(self) -> bool: ...
    def time_to_report(self) -> Literal[True] | None: ...
    def increment_request_count(self) -> None: ...
    def increment_borrow_count(self) -> None: ...
    def increment_sampled_count(self) -> None: ...
    @property
    def rate(self): ...
    @rate.setter
    def rate(self, v) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def priority(self): ...
    @property
    def reservoir(self) -> Reservoir: ...
    @reservoir.setter
    def reservoir(self, v: Reservoir) -> None: ...
    @property
    def can_borrow(self) -> bool: ...
    @property
    def request_count(self) -> int: ...
    @property
    def borrow_count(self) -> int: ...
    @property
    def sampled_count(self) -> int: ...



from typing import Final

XK_Hangul: Final = 0xFF31
XK_Hangul_Start: Final = 0xFF32
XK_Hangul_End: Final = 0xFF33
XK_Hangul_Hanja: Final = 0xFF34
XK_Hangul_Jamo: Final = 0xFF35
XK_Hangul_Romaja: Final = 0xFF36
XK_Hangul_Codeinput: Final = 0xFF37
XK_Hangul_Jeonja: Final = 0xFF38
XK_Hangul_Banja: Final = 0xFF39
XK_Hangul_PreHanja: Final = 0xFF3A
XK_Hangul_PostHanja: Final = 0xFF3B
XK_Hangul_SingleCandidate: Final = 0xFF3C
XK_Hangul_MultipleCandidate: Final = 0xFF3D
XK_Hangul_PreviousCandidate: Final = 0xFF3E
XK_Hangul_Special: Final = 0xFF3F
XK_Hangul_switch: Final = 0xFF7E
XK_Hangul_Kiyeog: Final = 0xEA1
XK_Hangul_SsangKiyeog: Final = 0xEA2
XK_Hangul_KiyeogSios: Final = 0xEA3
XK_Hangul_Nieun: Final = 0xEA4
XK_Hangul_NieunJieuj: Final = 0xEA5
XK_Hangul_NieunHieuh: Final = 0xEA6
XK_Hangul_Dikeud: Final = 0xEA7
XK_Hangul_SsangDikeud: Final = 0xEA8
XK_Hangul_Rieul: Final = 0xEA9
XK_Hangul_RieulKiyeog: Final = 0xEAA
XK_Hangul_RieulMieum: Final = 0xEAB
XK_Hangul_RieulPieub: Final = 0xEAC
XK_Hangul_RieulSios: Final = 0xEAD
XK_Hangul_RieulTieut: Final = 0xEAE
XK_Hangul_RieulPhieuf: Final = 0xEAF
XK_Hangul_RieulHieuh: Final = 0xEB0
XK_Hangul_Mieum: Final = 0xEB1
XK_Hangul_Pieub: Final = 0xEB2
XK_Hangul_SsangPieub: Final = 0xEB3
XK_Hangul_PieubSios: Final = 0xEB4
XK_Hangul_Sios: Final = 0xEB5
XK_Hangul_SsangSios: Final = 0xEB6
XK_Hangul_Ieung: Final = 0xEB7
XK_Hangul_Jieuj: Final = 0xEB8
XK_Hangul_SsangJieuj: Final = 0xEB9
XK_Hangul_Cieuc: Final = 0xEBA
XK_Hangul_Khieuq: Final = 0xEBB
XK_Hangul_Tieut: Final = 0xEBC
XK_Hangul_Phieuf: Final = 0xEBD
XK_Hangul_Hieuh: Final = 0xEBE
XK_Hangul_A: Final = 0xEBF
XK_Hangul_AE: Final = 0xEC0
XK_Hangul_YA: Final = 0xEC1
XK_Hangul_YAE: Final = 0xEC2
XK_Hangul_EO: Final = 0xEC3
XK_Hangul_E: Final = 0xEC4
XK_Hangul_YEO: Final = 0xEC5
XK_Hangul_YE: Final = 0xEC6
XK_Hangul_O: Final = 0xEC7
XK_Hangul_WA: Final = 0xEC8
XK_Hangul_WAE: Final = 0xEC9
XK_Hangul_OE: Final = 0xECA
XK_Hangul_YO: Final = 0xECB
XK_Hangul_U: Final = 0xECC
XK_Hangul_WEO: Final = 0xECD
XK_Hangul_WE: Final = 0xECE
XK_Hangul_WI: Final = 0xECF
XK_Hangul_YU: Final = 0xED0
XK_Hangul_EU: Final = 0xED1
XK_Hangul_YI: Final = 0xED2
XK_Hangul_I: Final = 0xED3
XK_Hangul_J_Kiyeog: Final = 0xED4
XK_Hangul_J_SsangKiyeog: Final = 0xED5
XK_Hangul_J_KiyeogSios: Final = 0xED6
XK_Hangul_J_Nieun: Final = 0xED7
XK_Hangul_J_NieunJieuj: Final = 0xED8
XK_Hangul_J_NieunHieuh: Final = 0xED9
XK_Hangul_J_Dikeud: Final = 0xEDA
XK_Hangul_J_Rieul: Final = 0xEDB
XK_Hangul_J_RieulKiyeog: Final = 0xEDC
XK_Hangul_J_RieulMieum: Final = 0xEDD
XK_Hangul_J_RieulPieub: Final = 0xEDE
XK_Hangul_J_RieulSios: Final = 0xEDF
XK_Hangul_J_RieulTieut: Final = 0xEE0
XK_Hangul_J_RieulPhieuf: Final = 0xEE1
XK_Hangul_J_RieulHieuh: Final = 0xEE2
XK_Hangul_J_Mieum: Final = 0xEE3
XK_Hangul_J_Pieub: Final = 0xEE4
XK_Hangul_J_PieubSios: Final = 0xEE5
XK_Hangul_J_Sios: Final = 0xEE6
XK_Hangul_J_SsangSios: Final = 0xEE7
XK_Hangul_J_Ieung: Final = 0xEE8
XK_Hangul_J_Jieuj: Final = 0xEE9
XK_Hangul_J_Cieuc: Final = 0xEEA
XK_Hangul_J_Khieuq: Final = 0xEEB
XK_Hangul_J_Tieut: Final = 0xEEC
XK_Hangul_J_Phieuf: Final = 0xEED
XK_Hangul_J_Hieuh: Final = 0xEEE
XK_Hangul_RieulYeorinHieuh: Final = 0xEEF
XK_Hangul_SunkyeongeumMieum: Final = 0xEF0
XK_Hangul_SunkyeongeumPieub: Final = 0xEF1
XK_Hangul_PanSios: Final = 0xEF2
XK_Hangul_KkogjiDalrinIeung: Final = 0xEF3
XK_Hangul_SunkyeongeumPhieuf: Final = 0xEF4
XK_Hangul_YeorinHieuh: Final = 0xEF5
XK_Hangul_AraeA: Final = 0xEF6
XK_Hangul_AraeAE: Final = 0xEF7
XK_Hangul_J_PanSios: Final = 0xEF8
XK_Hangul_J_KkogjiDalrinIeung: Final = 0xEF9
XK_Hangul_J_YeorinHieuh: Final = 0xEFA
XK_Korean_Won: Final = 0xEFF



from _typeshed import Incomplete

def create_client_assertion_jwt(
    domain: str, client_id: str, client_assertion_signing_key: str, client_assertion_signing_alg: str | None
) -> str: ...
def add_client_authentication(
    payload: dict[str, Incomplete],
    domain: str,
    client_id: str,
    client_secret: str | None,
    client_assertion_signing_key: str | None,
    client_assertion_signing_alg: str | None,
) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete

class DeviceAuthorizationEndpoint:
    ENDPOINT_NAME: str
    CLIENT_AUTH_METHODS: Incomplete
    USER_CODE_TYPE: str
    EXPIRES_IN: int
    INTERVAL: int
    server: Incomplete
    def __init__(self, server) -> None: ...
    def __call__(self, request): ...
    def create_endpoint_request(self, request): ...
    def authenticate_client(self, request): ...
    def create_endpoint_response(self, request): ...
    def generate_user_code(self): ...
    def generate_device_code(self): ...
    def get_verification_uri(self) -> None: ...
    def save_device_credential(self, client_id, scope, data) -> None: ...

def create_string_user_code(): ...
def create_digital_user_code(): ...



from braintree.graphql.types.customer_recommendations_payload import (
    CustomerRecommendationsPayload as CustomerRecommendationsPayload,
)
from braintree.graphql.types.payment_options import PaymentOptions as PaymentOptions
from braintree.graphql.types.payment_recommendation import PaymentRecommendation as PaymentRecommendation



from _typeshed import Incomplete

from antlr4.atn.ATNState import ATNState as ATNState, DecisionState as DecisionState
from antlr4.atn.ATNType import ATNType as ATNType
from antlr4.IntervalSet import IntervalSet as IntervalSet
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token

class ATN:
    __slots__ = (
        "grammarType",
        "maxTokenType",
        "states",
        "decisionToState",
        "ruleToStartState",
        "ruleToStopState",
        "modeNameToStartState",
        "ruleToTokenType",
        "lexerActions",
        "modeToStartState",
    )
    INVALID_ALT_NUMBER: int
    grammarType: Incomplete
    maxTokenType: Incomplete
    states: Incomplete
    decisionToState: Incomplete
    ruleToStartState: Incomplete
    ruleToStopState: Incomplete
    modeNameToStartState: Incomplete
    ruleToTokenType: Incomplete
    lexerActions: Incomplete
    modeToStartState: Incomplete
    def __init__(self, grammarType: ATNType, maxTokenType: int) -> None: ...
    def nextTokensInContext(self, s: ATNState, ctx: RuleContext): ...
    def nextTokensNoContext(self, s: ATNState): ...
    def nextTokens(self, s: ATNState, ctx: RuleContext | None = None): ...
    def addState(self, state: ATNState): ...
    def removeState(self, state: ATNState): ...
    def defineDecisionState(self, s: DecisionState): ...
    def getDecisionState(self, decision: int): ...
    def getExpectedTokens(self, stateNumber: int, ctx: RuleContext): ...



from _typeshed import Incomplete
from typing import ClassVar

from authlib.jose.rfc7517 import AsymmetricKey
from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKey, EllipticCurvePublicKey

class ECKey(AsymmetricKey):
    kty: str
    DSS_CURVES: Incomplete
    CURVES_DSS: Incomplete
    REQUIRED_JSON_FIELDS: ClassVar[list[str]]
    PUBLIC_KEY_FIELDS = REQUIRED_JSON_FIELDS
    PRIVATE_KEY_FIELDS: ClassVar[list[str]]
    PUBLIC_KEY_CLS: ClassVar[type]
    PRIVATE_KEY_CLS: ClassVar[type]
    SSH_PUBLIC_PREFIX: ClassVar[bytes]
    def exchange_shared_key(self, pubkey): ...
    @property
    def curve_key_size(self): ...
    def load_private_key(self) -> EllipticCurvePrivateKey: ...
    def load_public_key(self) -> EllipticCurvePublicKey: ...
    def dumps_private_key(self) -> dict[str, str]: ...
    def dumps_public_key(self) -> dict[str, str]: ...
    @classmethod
    def generate_key(cls, crv: str = "P-256", options=None, is_private: bool = False) -> ECKey: ...



from .assertion import client_secret_jwt_sign as client_secret_jwt_sign, private_key_jwt_sign as private_key_jwt_sign
from .auth import ClientSecretJWT as ClientSecretJWT, PrivateKeyJWT as PrivateKeyJWT
from .client import JWTBearerClientAssertion as JWTBearerClientAssertion
from .jwt_bearer import JWTBearerGrant as JWTBearerGrant
from .token import JWTBearerTokenGenerator as JWTBearerTokenGenerator
from .validator import JWTBearerToken as JWTBearerToken, JWTBearerTokenValidator as JWTBearerTokenValidator

__all__ = [
    "JWTBearerGrant",
    "JWTBearerClientAssertion",
    "client_secret_jwt_sign",
    "private_key_jwt_sign",
    "ClientSecretJWT",
    "PrivateKeyJWT",
    "JWTBearerToken",
    "JWTBearerTokenGenerator",
    "JWTBearerTokenValidator",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.compiler.xla.service.hlo_pb2
import tensorflow.tsl.protobuf.error_codes_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CompilationResultProto(google.protobuf.message.Message):
    """Describes the result of a TPU compilation. This is also used as TPU
    compilation result status payload.
    URI: "type.googleapis.com/tensorflow.tpu.CompilationResultProto"
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ErrorCode:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ErrorCodeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CompilationResultProto._ErrorCode.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: CompilationResultProto._ErrorCode.ValueType  # 0
        OUT_OF_MEMORY: CompilationResultProto._ErrorCode.ValueType  # 1

    class ErrorCode(_ErrorCode, metaclass=_ErrorCodeEnumTypeWrapper): ...
    UNKNOWN: CompilationResultProto.ErrorCode.ValueType  # 0
    OUT_OF_MEMORY: CompilationResultProto.ErrorCode.ValueType  # 1

    STATUS_CODE_FIELD_NUMBER: builtins.int
    STATUS_ERROR_MESSAGE_FIELD_NUMBER: builtins.int
    HLO_PROTOS_FIELD_NUMBER: builtins.int
    ERROR_CODE_FIELD_NUMBER: builtins.int
    status_code: tensorflow.tsl.protobuf.error_codes_pb2.Code.ValueType
    """The error message, if any, returned during compilation."""
    status_error_message: builtins.str
    error_code: global___CompilationResultProto.ErrorCode.ValueType
    @property
    def hlo_protos(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.service.hlo_pb2.HloProto]:
        """HLO proto."""

    def __init__(
        self,
        *,
        status_code: tensorflow.tsl.protobuf.error_codes_pb2.Code.ValueType | None = ...,
        status_error_message: builtins.str | None = ...,
        hlo_protos: collections.abc.Iterable[tensorflow.compiler.xla.service.hlo_pb2.HloProto] | None = ...,
        error_code: global___CompilationResultProto.ErrorCode.ValueType | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "error_code",
            b"error_code",
            "hlo_protos",
            b"hlo_protos",
            "status_code",
            b"status_code",
            "status_error_message",
            b"status_error_message",
        ],
    ) -> None: ...

global___CompilationResultProto = CompilationResultProto



from _typeshed import Incomplete

from reportlab.graphics.barcode.code39 import Standard39

class BaseLTOLabel(Standard39):
    LABELWIDTH: Incomplete
    LABELHEIGHT: Incomplete
    LABELROUND: Incomplete
    CODERATIO: float
    CODENOMINALWIDTH: Incomplete
    CODEBARHEIGHT: Incomplete
    CODEBARWIDTH: Incomplete
    CODEGAP = CODEBARWIDTH
    CODELQUIET: Incomplete
    CODERQUIET: Incomplete
    height: Incomplete
    border: Incomplete
    label: Incomplete
    def __init__(
        self, prefix: str = "", number=None, subtype: str = "1", border=None, checksum: bool = False, availheight=None
    ) -> None: ...
    def drawOn(self, canvas, x, y) -> None: ...

class VerticalLTOLabel(BaseLTOLabel):
    LABELFONT: Incomplete
    BLOCKWIDTH: Incomplete
    BLOCKHEIGHT: Incomplete
    LINEWIDTH: float
    NBBLOCKS: int
    COLORSCHEME: Incomplete
    colored: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def drawOn(self, canvas, x, y) -> None: ...

def test() -> None: ...



from _typeshed import ConvertibleToInt, Incomplete
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Alias, Bool, NoneSet, Typed, _ConvertibleToBool
from openpyxl.descriptors.nested import NestedText
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.connector import Shape
from openpyxl.drawing.graphic import GraphicFrame, GroupShape
from openpyxl.drawing.picture import PictureFrame
from openpyxl.drawing.xdr import XDRPoint2D, XDRPositiveSize2D

_TwoCellAnchorEditAs: TypeAlias = Literal["twoCell", "oneCell", "absolute"]

class AnchorClientData(Serialisable):
    fLocksWithSheet: Bool[Literal[True]]
    fPrintsWithSheet: Bool[Literal[True]]
    def __init__(
        self, fLocksWithSheet: _ConvertibleToBool | None = None, fPrintsWithSheet: _ConvertibleToBool | None = None
    ) -> None: ...

class AnchorMarker(Serialisable):
    tagname: ClassVar[str]
    col: NestedText[int, Literal[False]]
    colOff: NestedText[int, Literal[False]]
    row: NestedText[int, Literal[False]]
    rowOff: NestedText[int, Literal[False]]
    def __init__(
        self, col: ConvertibleToInt = 0, colOff: ConvertibleToInt = 0, row: ConvertibleToInt = 0, rowOff: ConvertibleToInt = 0
    ) -> None: ...

class _AnchorBase(Serialisable):
    sp: Typed[Shape, Literal[True]]
    shape: Alias
    grpSp: Typed[GroupShape, Literal[True]]
    groupShape: Alias
    graphicFrame: Typed[GraphicFrame, Literal[True]]
    cxnSp: Typed[Shape, Literal[True]]
    connectionShape: Alias
    pic: Typed[PictureFrame, Literal[True]]
    contentPart: Incomplete
    clientData: Typed[AnchorClientData, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        clientData: AnchorClientData | None = None,
        sp: Shape | None = None,
        grpSp: GroupShape | None = None,
        graphicFrame: GraphicFrame | None = None,
        cxnSp: Shape | None = None,
        pic: PictureFrame | None = None,
        contentPart=None,
    ) -> None: ...

class AbsoluteAnchor(_AnchorBase):
    tagname: ClassVar[str]
    pos: Typed[XDRPoint2D, Literal[False]]
    ext: Typed[XDRPositiveSize2D, Literal[False]]
    # Same as parent
    # sp = _AnchorBase.sp
    # grpSp = _AnchorBase.grpSp
    # graphicFrame = _AnchorBase.graphicFrame
    # cxnSp = _AnchorBase.cxnSp
    # pic = _AnchorBase.pic
    # contentPart = _AnchorBase.contentPart
    # clientData = _AnchorBase.clientData
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, pos: XDRPoint2D | None = None, ext: XDRPositiveSize2D | None = None, **kw) -> None: ...

class OneCellAnchor(_AnchorBase):
    tagname: ClassVar[str]
    _from: Typed[AnchorMarker, Literal[False]]  # Not private. Avoids name clash
    ext: Typed[XDRPositiveSize2D, Literal[False]]
    # Same as parent
    # sp = _AnchorBase.sp
    # grpSp = _AnchorBase.grpSp
    # graphicFrame = _AnchorBase.graphicFrame
    # cxnSp = _AnchorBase.cxnSp
    # pic = _AnchorBase.pic
    # contentPart = _AnchorBase.contentPart
    # clientData = _AnchorBase.clientData
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, _from: AnchorMarker | None = None, ext: XDRPositiveSize2D | None = None, **kw) -> None: ...

class TwoCellAnchor(_AnchorBase):
    tagname: ClassVar[str]
    editAs: NoneSet[_TwoCellAnchorEditAs]
    _from: Typed[AnchorMarker, Literal[False]]  # Not private. Avoids name clash
    to: Typed[AnchorMarker, Literal[False]]
    # Same as parent
    # sp = _AnchorBase.sp
    # grpSp = _AnchorBase.grpSp
    # graphicFrame = _AnchorBase.graphicFrame
    # cxnSp = _AnchorBase.cxnSp
    # pic = _AnchorBase.pic
    # contentPart = _AnchorBase.contentPart
    # clientData = _AnchorBase.clientData
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        editAs: _TwoCellAnchorEditAs | Literal["none"] | None = None,
        _from: AnchorMarker | None = None,
        to: AnchorMarker | None = None,
        **kw,
    ) -> None: ...

class SpreadsheetDrawing(Serialisable):
    tagname: ClassVar[str]
    mime_type: str
    PartName: str
    twoCellAnchor: Incomplete
    oneCellAnchor: Incomplete
    absoluteAnchor: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    charts: Incomplete
    images: Incomplete
    def __init__(self, twoCellAnchor=(), oneCellAnchor=(), absoluteAnchor=()) -> None: ...
    def __hash__(self) -> int: ...
    def __bool__(self) -> bool: ...
    @property
    def path(self) -> str: ...



from logging import Logger

from .decorator import cross_origin as cross_origin
from .extension import CORS as CORS
from .version import __version__ as __version__

rootlogger: Logger

__all__ = ["CORS", "__version__", "cross_origin"]



from collections.abc import Callable, Iterable
from typing import Protocol, type_check_only

from ..base import _ConvertibleToGeoSeries
from ..geodataframe import GeoDataFrame

@type_check_only
class _GeoCoder(Protocol):
    # Represents a geopy.geocoders.base.GeoCoder subclass without actually depending on geopy
    def geocode(self, query: str, /): ...
    def reverse(self, coords, /, exactly_one: bool = ...): ...

# TODO: Use something like `provider: Callable[P, _GeoCoder], **kwargs: P.kwargs` in the functions
# below if this ever becomes a thing
def geocode(strings: Iterable[str], provider: str | Callable[..., _GeoCoder] | None = None, **kwargs) -> GeoDataFrame: ...
def reverse_geocode(
    points: _ConvertibleToGeoSeries, provider: str | Callable[..., _GeoCoder] | None = None, **kwargs
) -> GeoDataFrame: ...



from braintree.braintree_gateway import BraintreeGateway
from braintree.configuration import Configuration
from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class PayPalPaymentResourceGateway:
    config: Configuration
    gateway: BraintreeGateway
    def __init__(self, gateway: BraintreeGateway) -> None: ...
    def update(self, params) -> SuccessfulResult | ErrorResult: ...



import sys
from datetime import datetime
from typing import Any, Callable, TypeVar

if sys.version_info[:2] < (3, 10):
    from typing_extensions import ParamSpec
else:
    from typing import ParamSpec


def validate_datetime(v, _):
    if not isinstance(v, datetime):
        raise Exception(f"Expected datetime, got {v}")
    return v


T = TypeVar("T")
P = ParamSpec("P")


def wrap(_: Callable[P, Any]) -> Callable[[Callable[..., T]], Callable[P, T]]:
    """Wrap a `Converter` `__init__` in a type-safe way."""

    def impl(x: Callable[..., T]) -> Callable[P, T]:
        return x

    return impl



import datetime
from typing import ClassVar, Final

from ..core import WesternCalendar

class Brazil(WesternCalendar):
    include_sao_jose: ClassVar[bool]
    sao_jose_label: ClassVar[str]
    include_sao_pedro: ClassVar[bool]
    sao_pedro_label: ClassVar[str]
    include_sao_joao: ClassVar[bool]
    sao_joao_label: ClassVar[str]
    include_servidor_publico: ClassVar[bool]
    servidor_publico_label: ClassVar[str]
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_day: ClassVar[tuple[int, int]]
    consciencia_negra_label: ClassVar[str]

class BrazilAcre(Brazil): ...
class BrazilAlagoas(Brazil): ...
class BrazilAmapa(Brazil): ...
class BrazilAmazonas(Brazil): ...
class BrazilBahia(Brazil): ...
class BrazilCeara(Brazil): ...
class BrazilDistritoFederal(Brazil): ...
class BrazilEspiritoSanto(Brazil): ...
class BrazilGoias(Brazil): ...
class BrazilMaranhao(Brazil): ...
class BrazilMinasGerais(Brazil): ...
class BrazilMatoGrosso(Brazil): ...
class BrazilMatoGrossoDoSul(Brazil): ...
class BrazilPara(Brazil): ...
class BrazilParaiba(Brazil): ...
class BrazilPernambuco(Brazil): ...
class BrazilPiaui(Brazil): ...
class BrazilParana(Brazil): ...

class BrazilRioDeJaneiro(Brazil):
    fat_tuesday_label: ClassVar[str]
    def get_dia_do_comercio(self, year: int) -> datetime.date | None: ...

class BrazilRioGrandeDoNorte(Brazil): ...
class BrazilRioGrandeDoSul(Brazil): ...
class BrazilRondonia(Brazil): ...
class BrazilRoraima(Brazil): ...
class BrazilSantaCatarina(Brazil): ...
class BrazilSaoPauloState(Brazil): ...

class BrazilSaoPauloCity(BrazilSaoPauloState):
    fat_tuesday_label: ClassVar[str]

class BrazilSergipe(Brazil): ...
class BrazilTocantins(Brazil): ...
class BrazilVitoriaCity(BrazilEspiritoSanto): ...
class BrazilVilaVelhaCity(BrazilEspiritoSanto): ...
class BrazilCariacicaCity(BrazilEspiritoSanto): ...
class BrazilGuarapariCity(BrazilEspiritoSanto): ...

class BrazilSerraCity(BrazilEspiritoSanto):
    fat_tuesday_label: ClassVar[str]

class BrazilRioBrancoCity(BrazilAcre): ...
class BrazilMaceioCity(BrazilAlagoas): ...
class BrazilManausCity(BrazilAmazonas): ...
class BrazilMacapaCity(BrazilAmapa): ...
class BrazilSalvadorCity(BrazilBahia): ...
class BrazilFortalezaCity(BrazilCeara): ...
class BrazilGoianiaCity(BrazilGoias): ...
class BrazilBeloHorizonteCity(BrazilMinasGerais): ...
class BrazilCampoGrandeCity(BrazilMatoGrossoDoSul): ...
class BrazilCuiabaCity(BrazilMatoGrosso): ...
class BrazilBelemCity(BrazilPara): ...
class BrazilJoaoPessoaCity(BrazilParaiba): ...
class BrazilRecifeCity(BrazilPernambuco): ...
class BrazilTeresinaCity(BrazilPiaui): ...
class BrazilCuritibaCity(BrazilParana): ...
class BrazilNatalCity(BrazilRioGrandeDoNorte): ...
class BrazilPortoVelhoCity(BrazilRondonia): ...
class BrazilBoaVistaCity(BrazilRoraima): ...
class BrazilPortoAlegreCity(BrazilRioGrandeDoSul): ...
class BrazilChapecoCity(BrazilSantaCatarina): ...
class BrazilFlorianopolisCity(BrazilSantaCatarina): ...
class BrazilJoinvilleCity(BrazilSantaCatarina): ...
class BrazilAracajuCity(BrazilSergipe): ...
class BrazilSorocabaCity(BrazilSaoPauloState): ...
class BrazilPalmasCity(BrazilTocantins): ...

class BrazilBankCalendar(Brazil):
    fat_tuesday_label: ClassVar[str]
    def get_last_day_of_year_for_only_internal_bank_trans(self, year: int) -> datetime.date: ...

IBGE_TUPLE: Final[tuple[tuple[str, type[Brazil]], ...]]
IBGE_REGISTER: Final[dict[str, type[Brazil]]]



import sys
from _typeshed import SupportsWrite
from typing import Final

from . import timemachine as timemachine
from .biffh import (
    XL_CELL_BLANK as XL_CELL_BLANK,
    XL_CELL_BOOLEAN as XL_CELL_BOOLEAN,
    XL_CELL_DATE as XL_CELL_DATE,
    XL_CELL_EMPTY as XL_CELL_EMPTY,
    XL_CELL_ERROR as XL_CELL_ERROR,
    XL_CELL_NUMBER as XL_CELL_NUMBER,
    XL_CELL_TEXT as XL_CELL_TEXT,
    biff_text_from_num as biff_text_from_num,
    error_text_from_code as error_text_from_code,
)
from .book import Book as Book, colname as colname, open_workbook_xls as open_workbook_xls
from .formula import *
from .info import __VERSION__ as __VERSION__, __version__ as __version__
from .sheet import empty_cell as empty_cell
from .xldate import XLDateError as XLDateError, xldate_as_datetime as xldate_as_datetime, xldate_as_tuple as xldate_as_tuple

FILE_FORMAT_DESCRIPTIONS: Final[dict[str, str]]
ZIP_SIGNATURE: Final[bytes]
PEEK_SIZE: Final[int]

def inspect_format(path: str | None = None, content: bytes | None = None) -> str | None: ...
def open_workbook(
    filename: str | None = None,
    logfile: SupportsWrite[str] = sys.stdout,
    verbosity: int = 0,
    use_mmap: bool = True,
    file_contents: bytes | None = None,
    encoding_override: str | None = None,
    formatting_info: bool = False,
    on_demand: bool = False,
    ragged_rows: bool = False,
    ignore_workbook_corruption: bool = False,
) -> Book: ...
def dump(filename: str, outfile: SupportsWrite[str] = sys.stdout, unnumbered: bool = False) -> None: ...
def count_records(filename: str, outfile: SupportsWrite[str] = sys.stdout) -> None: ...
